
*** This file starts from the raw application, enrollment and test score files, constructs a merged data set and prepares the data for analysis. 
*** Last update: 2013-05-09

/* General specifications */

clear all
set more off, permanently

*global dir = "h:/Research/Exam_Schools"
*adopath ++ "h:/ado/plus"

global dir = "//bbkinghome/rokkanen/Research/Exam_Schools"
adopath ++ "//bbkinghome/rokkanen/ado/plus"


/***************************************/
/* PREPARATION OF THE APPLICATION DATA */
/***************************************/


/* Open log file */

capture: log close
log using "$dir/Results_and_Notes/application_data.log", replace

/* Load data */

use "$dir/Data/exam_app.dta", clear
desc
compress
count

/* Destring variables */

replace reg_num = "" if reg_num == "NULL"
destring *, replace

/* Check the values of grade variable and rename it */

tab grade, miss
rename grade app_grade

/* Generate a dummy variable for gender ### WHAT ARE THE WEIRD VALUES? ### */

tab sex, miss
replace sex = "" if (sex != "F" & sex != "M") 
gen female_a = (sex == "F") if missing(sex) == 0
tab female_a, miss

/* Generate dummy variables for race ### VALUE 8 CODED AS MISSING ### */

tab race, miss
destring race, replace force
replace race = . if race == 8
replace race = 5 if race == 4
tab race, miss

gen black_a = (race == 1 | race == 7) if race != .
gen white_a = (race == 2) if race != .
gen asian_a = (race == 3) if race != .
gen hisp_a = (race == 4 | race == 5) if race != .
gen other_a = (race == 6) if race != .

tab black_a, miss
tab white_a, miss
tab asian_a, miss
tab hisp_a, miss
tab other_a, miss

/* Generate a dummy variable for applying from a private school */

tab curr_sch, miss
gen temp = real(substr(string(curr_sch), 1, 1))
gen private = (temp > 4) if temp != .
drop temp
tab schyear private, miss

/* Generate a dummy variable for Advanced Work Class ### ONLY FOR SOME YEARS? ### */

tab schyear awc0, miss
gen awc = (awc0 == "AW")
tab schyear awc, miss

/* Generate dummy variables for applying to a certain exam school ### WHAT ARE THOSE OBSERVATIONS THAT DID NOT APPLY TO EXAM SCHOOL AT ALL? ###*/

tab ch1, miss
tab ch2, miss
tab ch3, miss

gen app_bls = (ch1 == "B" | ch2 == "B" | ch3 == "B")
gen app_bla = (ch1 == "A" | ch2 == "A" | ch3 == "A")
gen app_obr = (ch1 == "T" | ch2 == "T" | ch3 == "T")
gen app_exam = (app_bls == 1 | app_bla == 1 | app_obr == 1)

tab schyear app_bls, miss
tab schyear app_bla, miss
tab schyear app_obr, miss
tab schyear app_exam, miss

/* Generate variables for the preferences of the applicants regarding the schools */

gen obr_pref = .
replace obr_pref = 1 if ch1 == "T"
replace obr_pref = 2 if ch2 == "T"
replace obr_pref = 3 if ch3 == "T"

gen bla_pref = .
replace bla_pref = 1 if ch1 == "A"
replace bla_pref = 2 if ch2 == "A"
replace bla_pref = 3 if ch3 == "A"

gen bls_pref = .
replace bls_pref = 1 if ch1 == "B"
replace bls_pref = 2 if ch2 == "B"
replace bls_pref = 3 if ch3 == "B"

tab obr_pref, miss
tab bla_pref, miss
tab bls_pref, miss

/* Generate variable for application preferences */

egen app_prefs = group(obr_pref bla_pref bls_pref), missing

/* Generate dummy variables for getting an offer from a certain school and for getting an offer altogether */

tab inv_sch, miss

gen offer_yn = (inv_sch == "1010" | inv_sch == "1020" | inv_sch == "1030")
gen offer_bls = (inv_sch == "1010")
gen offer_bla = (inv_sch == "1020")
gen offer_obr = (inv_sch == "1030")

tab schyear offer_yn, miss
tab schyear offer_bls, miss
tab schyear offer_bla, miss
tab schyear offer_obr, miss

/* Compute the ISEE score as a sum of all scores */

su verb quan read math, det
gen isee = verb + quan + read + math
su isee, det

/* Compute the mean and standard deviation of the GPAs and ISEE test scores in different year/grade combinations */

foreach x in gpa isee verb quan read math {
	bysort schyear app_grade: egen mean_`x' = mean(`x')
	bysort schyear app_grade: egen std_`x'= sd(`x')
	gen `x'_score = (`x' - mean_`x')/std_`x'
	drop mean_* std_*
}	

/* Recode value 0 to be missing for certain variables */

destring obr_comp bla_comp bls_comp, replace force
recode obr_rank bla_rank bls_rank obr_comp bla_comp bls_comp verb quan read math gpa (0 = .)

/* Compute cutoffs for each school */

foreach sch in obr bla bls {
	bysort schyear app_grade: egen temp = max(`sch'_rank) if offer_`sch' == 1
	bysort schyear app_grade: egen `sch'_cutoff = mean(temp)
	drop temp
	tab `sch'_cutoff, miss
	tab schyear app_grade if `sch'_cutoff == ., miss
}

/* Generate indicators for whether an applicant clears the cutoff for his 1st, 2nd and 3rd choice */

forval i = 1/3 {
	gen above_choice`i' = 0
	foreach sch in obr bla bls {
		replace above_choice`i' = 1 if `sch'_pref == `i' & `sch'_rank <= `sch'_cutoff 
	}
}

/* Define the standardized running variable to be the percentile rank and recenter it to be 0 at the cutoff */

foreach sch in obr bla bls {
	bysort schyear app_grade: egen temp_n = count(`sch'_rank)
	bysort schyear app_grade: egen temp_i = rank(-`sch'_rank), track
	gen runvar_`sch' = 100*(temp_i - 1)/(temp_n - 1)
	drop temp_*
	su runvar_`sch'
}

foreach sch in obr bla bls {	
	bysort schyear app_grade: egen temp1 = min(runvar_`sch') if offer_`sch' == 1
	bysort schyear app_grade: egen temp2 = mean(temp1)
	replace runvar_`sch' = runvar_`sch' - temp2
	drop temp*
}

/* Generate composite scores for each application cohort */

tab schyear app_grade if obr_comp != .
tab schyear app_grade if bla_comp != .
tab schyear app_grade if bls_comp != .

foreach sch in obr bla bls {
	mat results_`sch' = J(1, 2, .)
	foreach yr in 1995 1996 1997 1998 1999 2000 2009 {
		mat row = J(1, 1, .)
		foreach gr in 7 9 {
			disp "`yr' - `gr'"
			reg `sch'_comp verb quan read math gpa if schyear == `yr' & app_grade == `gr'
			mat row = (row, round(e(r2), 0.001))
			mat rownames row = `yr'
		}
		mat results_`sch' = (results_`sch'\ row[1..., 2...])
	}
	mat colnames results_`sch' = 7 9
	mat results_`sch' = results_`sch'[2..., 1...]
	mat list results_`sch'
}

foreach sch in obr bla bls {
	bysort schyear app_grade: egen temp_n = count(`sch'_rank)
	bysort schyear app_grade: egen temp_i = rank(-`sch'_rank), track
	gen temp_p = temp_i/(temp_n + 1)
	su temp_p
	gen `sch'_gencomp = invnormal(temp_p)
	drop temp_*
	mat results_`sch' = J(1, 2, .)
	forval yr = 1995/2009 {
		mat row = J(1, 1, .)
		foreach gr in 7 9 {
			quietly: reg `sch'_gencomp verb quan read math gpa if schyear == `yr' & app_grade == `gr'
			mat row = (row, round(e(r2), 0.001))
			mat rownames row = `yr'
		}
		mat results_`sch' = (results_`sch'\ row[1..., 2...])
	}
	mat colnames results_`sch' = 7 9
	mat results_`sch' = results_`sch'[2..., 1...]
}
mat list results_obr
mat list results_bla
mat list results_bls

/* Standardize the generated composite scores */

foreach sch in obr bla bls {
	bysort schyear app_grade: egen temp1 = mean(`sch'_gencomp)
	bysort schyear app_grade: egen temp2 = sd(`sch'_gencomp)
	gen `sch'_stdcomp = (`sch'_gencomp - temp1)/temp2
	drop temp*
}

/* Center the standardized composite scores to be 0 at the cutoffs */

foreach sch in obr bla bls {
	bysort schyear app_grade: egen temp1 = min(`sch'_stdcomp) if offer_`sch' == 1
	bysort schyear app_grade: egen temp2 = min(temp1)
	replace `sch'_stdcomp = `sch'_stdcomp - temp2
	drop temp*
}

/* Compute affirmative action cutoffs for each school in terms of the actual/generated composites */

foreach sch in obr bla bls {
	foreach x in comp gencomp {
		bysort schyear app_grade race: egen temp = min(`sch'_`x') if offer_`sch' == 1
		bysort schyear app_grade race: egen `sch'_aacutoff_`x' = mean(temp)
		drop temp
		tab `sch'_aacutoff_`x', miss
		bysort app_grade: tab schyear race if `sch'_aacutoff_`x' == ., miss
	}
}

/* Check number of observations before any restrictions */

tab schyear if app_grade == 7, miss matcell(temp)
mat kuv_7 = temp
tab schyear if app_grade == 9, miss matcell(temp)
mat kuv_9 = temp
tab schyear, miss matcell(temp)
mat kuv_all = temp
global columns = "ALL"

mat rownames kuv_7 = 1995 1996 1997 1998 1999 2000 2001 2002 2003 2004 2005 2006 2007 2008 2009
mat rownames kuv_9 = 1995 1996 1997 1998 1999 2000 2001 2002 2003 2004 2005 2006 2007 2008 2009
mat rownames kuv_all = 1995 1996 1997 1998 1999 2000 2001 2002 2003 2004 2005 2006 2007 2008 2009

/* Delete students who applied to grade 10 */

tab schyear app_grade, miss
drop if app_grade == 10

tab schyear if app_grade == 7, miss matcell(temp)
mat kuv_7 = (kuv_7, temp)
tab schyear if app_grade == 9, miss matcell(temp)
mat kuv_9 = (kuv_9, temp)
tab schyear, miss matcell(temp)
mat kuv_all = (kuv_all, temp)
global columns = "$columns" + " GR10"

/* Delete duplicate observations of a student */

duplicates tag *, gen(temp)
tab schyear temp, miss
duplicates drop *, force
drop temp

tab schyear if app_grade == 7, miss matcell(temp)
mat kuv_7 = (kuv_7, temp)
tab schyear if app_grade == 9, miss matcell(temp)
mat kuv_9 = (kuv_9, temp)
tab schyear, miss matcell(temp)
mat kuv_all = (kuv_all, temp)
global columns = "$columns" + " DUPL"

/* Delete private school applicants */

tab schyear private, miss
drop if private == 1

tab schyear if app_grade == 7, miss matcell(temp)
mat kuv_7 = (kuv_7, temp)
tab schyear if app_grade == 9, miss matcell(temp)
mat kuv_9 = (kuv_9, temp)
tab schyear, miss matcell(temp)
mat kuv_all = (kuv_all, temp)
global columns = "$columns" + " PRIVATE"

/* Extract student numbers, grades and years for Parag */

gsort schyear app_grade
outsheet schyear app_grade studentno using "$dir/Data/exam_applicants_for_parag.csv", comma replace

/* Delete students who didn't actually apply to exam schools */

tab schyear app_exam, miss
drop if app_exam != 1

tab schyear if app_grade == 7, miss matcell(temp)
mat kuv_7 = (kuv_7, temp)
tab schyear if app_grade == 9, miss matcell(temp)
mat kuv_9 = (kuv_9, temp)
tab schyear, miss matcell(temp)
mat kuv_all = (kuv_all, temp)
global columns = "$columns" + " APPLY"

/* Delete students that do not have a rank in any of the exam schools */

gen temp = ((bls_rank == 0 | bls_rank == .) & (bla_rank == 0 | bla_rank == .) & (obr_rank == 0 | obr_rank == .))
tab schyear temp, miss
drop if temp == 1
drop temp

tab schyear if app_grade == 7, miss matcell(temp)
mat kuv_7 = (kuv_7, temp)
tab schyear if app_grade == 9, miss matcell(temp)
mat kuv_9 = (kuv_9, temp)
tab schyear, miss matcell(temp)
mat kuv_all = (kuv_all, temp)
global columns = "$columns" + " RANK"

/* Delete students that get an offer from a school where they did not apply */

tab schyear app_grade if (app_obr == 0 & offer_obr == 1) | (app_bla == 0 & offer_bla == 1) | (app_bls == 0 & offer_bls == 1), miss
drop if (app_obr == 0 & offer_obr == 1) | (app_bla == 0 & offer_bla == 1) | (app_bls == 0 & offer_bls == 1)

tab schyear if app_grade == 7, miss matcell(temp)
mat kuv_7 = (kuv_7, temp)
tab schyear if app_grade == 9, miss matcell(temp)
mat kuv_9 = (kuv_9, temp)
tab schyear, miss matcell(temp)
mat kuv_all = (kuv_all, temp)
global columns = "$columns" + " NOAPPOFFER"

/* Check whether we have multiple observations of a student number within a given year  */

duplicates report schyear studentno

/* Check whether we have multiple observations of a registration number within a given year  */

duplicates report schyear reg_num

/* Generate a new id variable to be used to identify unsuccesfull matches in the merging */

gsort schyear studentno
gen merge_check_id = _n

/* Save the data */

count
compress
gsort schyear studentno
save "$dir/Data/application_data.dta", replace

/* Close log file */

log close


/**************************************/
/* PREPARATION OF THE ENROLLMENT DATA */
/**************************************/


/* Open log file */

capture: log close
log using "$dir/Results_and_Notes/enrollment_data.log", replace

/* Load data */

use "$dir/Data/enroll.dta", clear
desc
compress
count

/* Add the additional years 1995-1997 */

gen entcode2 = string(entcode)
drop entcode
rename entcode2 entcode

append using "$dir/Data/studentextract1995_97_complete.dta"
drop col20

/* Modify the year variable to match the other data files */

tab year, miss
gen schyear = real(substr(string(year), 2, .)) if length(string(year)) == 5
replace schyear = year if length(string(year)) == 4
tab schyear, miss
drop year

/* Drop redundant variables */

drop homelang1 homeroom entcode homelang2 homelang3 unknown1 unknown2 unknown3

/* Replace years 2005- from the new file received in August 2012 (change year to refer to spring) */

drop if schyear >= 2005
tostring *, replace
append using "$dir/Data/BPSEnrollmentAugust2012_MIIKKA2.dta"
destring schyear year, replace
replace schyear = year + 1 if missing(schyear) == 1
drop year
destring *, replace

/* Drop the duplicate year 1997 (same as 1998 in the old data) and recode 1995 to 1996 and 1996 to 1997 (so that they refer to spring term as in the old data)*/

drop if schyear == 1997
recode schyear (1995 = 1996) (1996 = 1997)
tab schyear, miss

/* Change the date of birth variable into Stata's date form */

gen dateofbirth3 = dateofbirth if schyear >= 2005
destring dateofbirth3, replace

gen temp = substr(dateofbirth, 1, 2) + " " + substr(dateofbirth, 3, 3)  + " " + substr(dateofbirth, 6, 4)
gen dob = date(temp, "DMY")

replace dob = dateofbirth2 if dateofbirth2 != .
replace dob = dateofbirth3 if dateofbirth3 != .

format dob %d
drop temp dateofbirth dateofbirth2 dateofbirth3

/* Generate a numerical grade variable */

tab grade, miss
replace grade = "" if grade == "218" | grade == "319" | grade == "442" | grade == "457" | grade == "DIS"
destring grade, force gen(numgrade)
replace numgrade = 0 if grade == "k0" | grade == "K0"
replace numgrade = 0.1 if grade == "k1" | grade == "K1"
replace numgrade = 0.2 if grade == "k2" | grade == "K2"
tab numgrade, miss
drop grade
rename numgrade grade

/* Create a file that contains schyear, studentno and school (schyear - 1 since year refers to fall in mcas file) */

save "$dir/Data/temporary_data.dta", replace

keep schyear grade studentno school
replace schyear = schyear - 1

duplicates tag studentno schyear, gen(temp1)
gsort schyear studentno
bysort schyear studentno: egen temp2 = sd(school) if temp1 != 0
tab temp2
drop if temp1 != 0
drop temp1 temp2

gsort schyear studentno
save "$dir/Data/school_data.dta", replace

use "$dir/Data/temporary_data.dta", clear

/* Generate a dummy variable for gender */

tab sex, miss
replace sex = "" if sex != "M" & sex != "F"
gen female = (sex == "F") if missing(sex) == 0
tab female, miss

/* Generate dummy variables for various races ### VALUE 8 CODED AS MISSING  ### */

tab race, miss
replace race = . if race == 8

gen black = (race == 1 | race == 7) if race != .
gen white = (race == 2) if race != .
gen asian = (race == 3) if race != .
gen hisp = (race == 4 | race == 5) if race != .
gen other = (race == 6) if race != .

tab black, miss
tab white, miss
tab asian, miss
tab hisp, miss
tab other, miss

/* Generate dummy for free meal (assuming missing means one does not get free meal) */

tab schyear food, miss
gen free = (food == "AF" | food == "AR" | food == "DC" | food == "FR" | food == "RF" | food == "TA")
tab schyear free, miss

/* Generate dummy for being bilingual ### PROBLEMS IN 1996-1997 ### */

rename bil biling
tab firstlang , miss
gen bil = (firstlang != "E" & firstlang != "NULL"  & firstlang != "")
tab schyear bil, miss
replace bil = . if (schyear <= 1997 | schyear >= 2005)
tab schyear bil, miss

/* Generate dummy for limited English proficiency ### PROBLEMS IN 1996-1997 ### */

tab schyear lepstatus, miss
gen lep = (lepstatus == "1" | lepstatus == "F" | lepstatus == "L")
tab schyear lep, miss
replace lep = . if schyear < 1998
tab schyear lep, miss

/* Generate dummy for being in need of special education ### ONLY AVAILABLE FROM YEAR 1998 ONWARDS ### */

tab sncode, miss
gen special = 1 - missing(sncode)
tab schyear special, miss
replace special = . if schyear < 1998
tab schyear special, miss

/* Generate a data with student characteristics by school/grade/year (change year to refer to fall) */

save "$dir/Data/temporary_data.dta", replace

keep schyear school grade female black white asian hisp other free bil lep special

gen blackhisp = (black == 1 | hisp == 1) if black != . & hisp != .

foreach x in blackhisp free lep special female {
	bysort schyear school grade: egen prop_`x' = mean(`x')
	bysort schyear school grade: egen prop_`x'_N = count(`x')
}

keep schyear school grade prop_*
duplicates drop schyear school grade, force

rename school next_school
rename grade app_grade
replace schyear = schyear - 1

gsort schyear app_grade next_school
drop if next_school == .
save "$dir/Data/student_characteristics_data.dta", replace

use "$dir/Data/temporary_data.dta", clear

/* Generate dummies for attending one of the exam schools (assuming missing means no) */

tab school, miss

gen in_bls = (school == 1010)
gen in_bla = (school == 1020)
gen in_obr = (school == 1030)
gen in_exam = (in_bls == 1 | in_bla == 1 | in_obr == 1)

tab in_bls, miss
tab in_bla, miss
tab in_obr, miss
tab in_exam, miss

/* Check whether we have multiple observations of a student number within a given grade (this is natural) */

duplicates report grade studentno

/* Check whether we have multiple observations of a student number within a given year (this might be a problem) */

duplicates report schyear studentno

/* Generate variable for total years spent in exam schools between 1996 and 2010 */

bysort studentno: egen yrs_in_bls_tot = total(in_bls == 1)
bysort studentno: egen yrs_in_bla_tot = total(in_bla == 1)
bysort studentno: egen yrs_in_obr_tot = total(in_obr == 1)
bysort studentno: egen yrs_in_exam_tot = total(in_exam == 1)

tab yrs_in_bls_tot, miss
tab yrs_in_bla_tot, miss
tab yrs_in_obr_tot, miss
tab yrs_in_exam_tot, miss

/* Generate variable for current years spent in exam schools from 1996 onwards */

gen yrs_in_bls_cur = 0
gen yrs_in_bla_cur = 0
gen yrs_in_obr_cur = 0
gen yrs_in_exam_cur = 0
forval yr = 1996/2012 {
	bysort studentno: egen temp = total(in_bls == 1) if schyear <= `yr'
	replace yrs_in_bls_cur = temp if schyear == `yr'
	drop temp

	bysort studentno: egen temp = total(in_bla == 1) if schyear <= `yr'
	replace yrs_in_bla_cur = temp if schyear == `yr'
	drop temp

	bysort studentno: egen temp = total(in_obr == 1) if schyear <= `yr'
	replace yrs_in_obr_cur = temp if schyear == `yr'
	drop temp

	bysort studentno: egen temp = total(in_exam == 1) if schyear <= `yr'
	replace yrs_in_exam_cur = temp if schyear == `yr'
	drop temp

	compress
}

tab yrs_in_bls_cur, miss
tab yrs_in_bla_cur, miss
tab yrs_in_obr_cur, miss
tab yrs_in_exam_cur, miss

/* Generate a variable for school and grade attended in a given year. If there are multiple observations in different schools code as missing. */

forval yr = 1996/2012 {
	bysort studentno: egen temp = mean(school) if schyear == `yr'
	bysort studentno: egen temp2 = sd(school) if schyear == `yr'
	bysort studentno: egen school_`yr' = mean(temp) if (temp2 == 0 | temp2 == .)
	drop temp temp2
	
	bysort studentno: egen temp = mean(grade) if schyear == `yr'
	bysort studentno: egen temp2 = sd(grade) if schyear == `yr'
	bysort studentno: egen grade_`yr' = mean(temp) if (temp2 == 0 | temp2 == .)
	drop temp temp2
	
	compress
}

/* Generate variables for years spent in exam schools until a given year */

forval yr = 1996/2012 {
	bysort studentno: egen temp = mean(yrs_in_exam_cur) if schyear == `yr'
	bysort studentno: egen yrs_in_exam_`yr' = mean(temp)
	drop temp
	tab yrs_in_exam_`yr', miss
	
	bysort studentno: egen temp = mean(yrs_in_obr_cur) if schyear == `yr'
	bysort studentno: egen yrs_in_obr_`yr' = mean(temp)
	drop temp
	tab yrs_in_obr_`yr', miss

	bysort studentno: egen temp = mean(yrs_in_bla_cur) if schyear == `yr'
	bysort studentno: egen yrs_in_bla_`yr' = mean(temp)
	drop temp
	tab yrs_in_bla_`yr', miss

	bysort studentno: egen temp = mean(yrs_in_bls_cur) if schyear == `yr'
	bysort studentno: egen yrs_in_bls_`yr' = mean(temp)
	drop temp
	tab yrs_in_bls_`yr', miss

	compress
}

/* Generate a data with enrollment numbers for each year-school-grade cell (make schyear refer to fall as in the test file) */

save "$dir/Data/enrollment_data.dta", replace

keep schyear school grade
bysort schyear school grade: gen enrollment = _N
duplicates drop schyear school grade, force
rename school next_school
rename grade app_grade
replace schyear = schyear - 1
drop if next_school == .
save "$dir/Data/peer_enrollment_data.dta", replace

use "$dir/Data/enrollment_data.dta", clear

/* Generate a data of 12th graders to be matched to NSC file */

save "$dir/Data/enrollment_data.dta", replace

keep if grade == 12
save "$dir/Data/12th_graders_enrollment_data.dta", replace


use "$dir/Data/enrollment_data.dta", clear

/* Keep only grade 6 and 8 */

tab grade, miss
keep if grade == 6 | grade == 8

/* Keep years 1996-2009 */

tab schyear, miss
keep if schyear >= 1996 & schyear <= 2009

/* Save data */

count
compress
sort studentno
save "$dir/Data/enrollment_data.dta", replace

/* Close log file */

log close


/****************************/
/* PREPARATION OF MCAS DATA */
/****************************/


/* Open log file */

capture: log close
log using "$dir/Results_and_Notes/mcas_data.log", replace

/* Load data */

use "$dir/Data/mcas.dta", clear
desc
compress
count

/* Check what tests do we have for each year */

tab testid schyear, miss

/* Keep only test scores for mathematics and English in grades 4, 7, 8 and 10 */

tab testid, miss
gen temp = 1 if testid == "MCAS04AE" | testid == "MCAS07AE" | testid == "MCAS08AE" | testid == "MCAS10AE"
replace temp = 1 if testid == "MCAS04AM" | testid == "MCAS07AM"  | testid == "MCAS08AM" | testid == "MCAS10AM" 
replace temp = 1 if testid == "MCAS08AS" | testid == "MCASHSAB" | testid == "MCASHSAC" | testid == "MCASHSAP" | testid == "MCASHSAT"
keep if temp == 1
drop temp

/* Check whether we have multiple observations of a student number and a test within a given year  */

duplicates report schyear studentno testid 

/* Compute the mean and standard deviation of the test scores for each year */

tab scaled, miss
destring scaled, replace force

gsort schyear testid
by schyear testid: egen mean = mean(scaled)
by schyear testid: egen std = sd(scaled)

tab mean, miss
tab std, miss

/* Standardize the test scores */

gen score = (scaled - mean)/std
su score, det

/* Keep only the relevant variables */

keep schyear studentno testid score scaled level

/* Scores for each test */

foreach test in eng4 eng7 eng8 eng10 math4 math7 math8 math10 scie8 biohs physhs chemhs techhs {
	if "`test'" == "eng4" {
		local testid = "MCAS04AE"
	}
	else if "`test'" == "eng7" {
		local testid = "MCAS07AE"
	}
	else if "`test'" == "eng8" {
		local testid = "MCAS08AE"
	}
	else if "`test'" == "eng10" {
		local testid = "MCAS10AE"
	}
	else if "`test'" == "math4" {
		local testid = "MCAS04AM"
	}
	else if "`test'" == "math7" {
		local testid = "MCAS07AM"
	}
	else if "`test'" == "math8" {
		local testid = "MCAS08AM"
	}
	else if "`test'" == "math10" {
		local testid = "MCAS10AM"
	}
	else if "`test'" == "scie8" {
		local testid = "MCAS08AS"
	}
	else if "`test'" == "biohs" {
		local testid = "MCASHSAB"
	}
	else if "`test'" == "physhs" {
		local testid = "MCASHSAP"
	}
	else if "`test'" == "chemhs" {
		local testid = "MCASHSAC"
	}	
	else if "`test'" == "techhs" {
		local testid = "MCASHSAT"
	}
	gsort schyear studentno
	gen temp = score if testid == "`testid'"
	by schyear studentno: egen `test' = mean(temp)
	drop temp
	gen temp = scaled if testid == "`testid'"
	by schyear studentno: egen `test'_scaled = mean(temp)
	drop temp
	gen yr_`test' = schyear if `test' != .
}

/* Remove duplicate year-student observations. These are the result of separate observations for each tests */

duplicates report schyear studentno, force
duplicates drop schyear studentno, force

/* Leave only the first test scores for students who have taken a test several times */

duplicates report testid studentno, force

foreach test in eng4 eng7 eng8 eng10 math4 math7 math8 math10 scie8 biohs physhs chemhs techhs {
	bysort studentno: egen min_yr = min(yr_`test')
	replace `test' = . if min_yr < yr_`test'
	replace `test'_scaled = . if min_yr < yr_`test'
	replace yr_`test' = . if min_yr < yr_`test'
	drop min_yr
}

/* Leave only one observation per student */

collapse (mean) *eng4* *eng7* *eng8* *eng10* *math4* *math7* *math8* *math10* *scie8* *biohs* *physhs* *chemhs* *techhs*, by(studentno)

foreach test in eng4 eng7 eng8 eng10 math4 math7 math8 math10 scie8 biohs physhs chemhs techhs {
	tab yr_`test', miss
}

/* Generate the achievement level variable */

foreach test in eng4 eng7 eng8 eng10 math4 math7 math8 math10 scie8 biohs physhs chemhs techhs {
	gen `test'_level = .
	replace `test'_level = 1 if `test'_scaled >= 260 & `test'_scaled <= 280
	replace `test'_level = 2 if `test'_scaled >= 240 & `test'_scaled <= 259
	replace `test'_level = 3 if `test'_scaled >= 220 & `test'_scaled <= 239
	replace `test'_level = 4 if `test'_scaled >= 200 & `test'_scaled <= 219

	su `test'_scaled if `test'_level == 1
	su `test'_scaled if `test'_level == 2
	su `test'_scaled if `test'_level == 3
	su `test'_scaled if `test'_level == 4

	rename `test'_scaled `test'raw
}

/* Generata data with mean scores by school, grade and year */

save "$dir/Data/temporary_data.dta", replace

use "$dir/Data/school_data.dta", clear
gsort studentno
save "$dir/Data/school_data.dta", replace

use "$dir/Data/temporary_data.dta", clear

gsort studentno
merge studentno using "$dir/Data/school_data.dta"

tab _merge, miss
drop if _merge != 3
drop _merge

foreach test in eng4 eng7 eng8 eng10 math4 math7 math8 math10 {
	gsort school grade schyear
	by school grade schyear: egen peer_`test' = mean(`test')
	by school grade schyear: egen peer_`test'_N = count(`test')
}

duplicates drop school grade schyear, force
keep school grade schyear peer_*

rename school next_school
rename grade app_grade

gsort schyear app_grade next_school
drop if next_school == .
save "$dir/Data/peer_mean_data.dta", replace

use "$dir/Data/temporary_data.dta", clear

/* Generata data with all (potential) the peers */

save "$dir/Data/temporary_data.dta", replace

use "$dir/Data/school_data.dta", clear
gsort studentno
save "$dir/Data/school_data.dta", replace

use "$dir/Data/temporary_data.dta", clear

gsort studentno
merge studentno using "$dir/Data/school_data.dta"

tab _merge, miss
drop if _merge != 3
drop _merge

keep studentno schyear school grade math4 eng4 math8 eng7 

rename school next_school
rename grade app_grade
foreach x in studentno math4 eng4 math8 eng7 {
	rename `x' peers_`x'
}
	
keep if (app_grade == 7 | app_grade == 9) & missing(schyear) == 0 & missing(next_school) == 0
compress
gsort schyear app_grade next_school
drop if next_school == .
save "$dir/Data/peers_data.dta", replace

use "$dir/Data/temporary_data.dta", clear

/* Generate variables for relative ranks in 7th and 9th grade in the baseline variables */

save "$dir/Data/temporary_data.dta", replace

use "$dir/Data/school_data.dta", clear
gsort studentno
save "$dir/Data/school_data.dta", replace

use "$dir/Data/temporary_data.dta", clear

gsort studentno
merge studentno using "$dir/Data/school_data.dta"

foreach test in eng4 eng7 math4 math7 math8 {
	gsort school grade schyear
	by school grade schyear: egen temp1 = rank(`test'), track
	by school grade schyear: egen temp2 = count(`test')
	gen pcrank_`test' = (temp1 - 1)/(temp2 - 1)
	drop temp1 temp2
}
compress

foreach test in eng4 eng7 math4 math8 {
	quietly: levelsof schyear, local(years)
	foreach yr in `years' {
		gen temp = pcrank_`test' if schyear == `yr'
		bysort studentno: egen pcrank`yr'_`test' = mean(temp)
		drop temp
	}
	
}
drop pcrank_* school grade _merge
compress
duplicates drop studentno, force

/* Save data */

compress
sort studentno
save "$dir/Data/test_score_data.dta", replace

/* Close log file */

log close


/***************************/
/* PREPARATION OF SAT DATA */
/***************************/


/* Open log file */

capture: log close
log using "$dir/Results_and_Notes/sat_data.log", replace

/* Generate empty data */

clear
gen satv = .
gen satm = .
gen satw = .
gen studentno = .
gen date = .
gen year = .
tostring *, replace
save "$dir/Data/sat_data.dta", replace

/* Add data from 2005 */

use "$dir/Data/SAT2005.dta", clear
desc
compress
count

gen year = 2005
gen date = latstsat
gen satv =  satvrecn
gen satm = satmrecn
gen satw = .

keep year date satv satm satw studentno
tostring *, replace
append using "$dir/Data/sat_data.dta"
count
save "$dir/Data/sat_data.dta", replace

/* Add data from 2006 */

use "$dir/Data/SAT2006.dta", clear
desc
compress
count

gen year = 2006
gen date = lasttestdate
gen satv =  vscr
gen satm = mscr
gen satw = wscr

keep year date satv satm satw studentno
tostring *, replace
append using "$dir/Data/sat_data.dta"
count
save "$dir/Data/sat_data.dta", replace

/* Add data from 2007 */

use "$dir/Data/SAT2007.dta", clear
desc
compress
count

gen year = 2007
gen date = .
gen satv =  sativ
gen satm = satim
gen satw = satiw

keep year date satv satm satw studentno
tostring *, replace
append using "$dir/Data/sat_data.dta"
count
save "$dir/Data/sat_data.dta", replace

/* Add data from 2008 */

use "$dir/Data/SAT2008.dta", clear
desc
compress
count

gen year = 2008
gen date = lateadm
gen satv =  sc1
gen satm = sc2
gen satw = sc3

keep year date satv satm satw studentno
tostring *, replace
append using "$dir/Data/sat_data.dta"
count
save "$dir/Data/sat_data.dta", replace

/* Add data from 2009 */

use "$dir/Data/SAT2009.dta", clear
desc
compress
count

gen year = 2009
gen date = lateadm
gen satv =  sativ
gen satm = satim
gen satw = satiw

keep year date satv satm satw studentno
tostring *, replace
append using "$dir/Data/sat_data.dta"
count
save "$dir/Data/sat_data.dta", replace

/* Add data from 2010 */

use "$dir/Data/SAT2010.dta", clear
desc
compress
count

gen year = 2010
gen date = lateadm
gen satv =  sativ
gen satm = satim
gen satw = satiw

keep year date satv satm satw studentno
tostring *, replace
append using "$dir/Data/sat_data.dta"
count
save "$dir/Data/sat_data.dta", replace

/* Get rid of erroneous observations */

foreach x in studentno date satv satm satw {
	drop if regexm(`x', "affected") == 1
}
destring *, replace
drop if missing(studentno)
drop if missing(satv) == 1 & missing(satm) == 1 & missing(satw) == 1

/* Recode scores from each year to be on the scale 200-800 */

bysort year: su satv satm satw
foreach x in satv satm satw {
	replace `x' = 10*`x' if `x' < 200
}
bysort year: su satv satm satw

/* Generate test date variable in a elapsed time form*/

tostring date, replace
tab date, miss
replace date = "0" + date if length(date) == 3
replace date = "" if date == "."
tab date, miss

gen testmonth = real(substr(date, 1, 2))
tab testmonth, miss

gen testyear = 2000 + real(substr(date, 3, 2))
tab testyear, miss

drop date
gen date = mdy(testmonth, 15, testyear)
format date %td 
drop testmonth testyear

/* Check the test date composition for each file year */

tab date year, miss

/* Change year to 2005 for kids in 2006 file without writing score as these scores are from 2004 */

tab date year if satw == ., miss
replace year = 2005 if satw == .

/* Generate a variable for total score (= math + verbal) */

egen sat = rowtotal(satv satm)
bysort year: su sat*

/* Drop duplicates where score is the same (keep the one with date or the one from earlier file) */

duplicates report studentno
duplicates report studentno satv satm satw
duplicates report studentno satv satm satw if date!=.
duplicates report studentno satv satm satw date

gsort date year
duplicates drop studentno satv satm satw, force

/* Leave either the first or the worse score for a kid who has multiple scores in same year */

duplicates tag studentno year, gen(dupl)
list if dupl == 1

gsort date sat
duplicates drop studentno year, force
list if dupl == 1
drop dupl

/* Standardize scores within file years */

gen satraw = sat
foreach x in satv satm satw sat {
	bysort year: egen mean = mean(`x')
	bysort year: egen sd = sd(`x')
	replace `x' = (`x' - mean)/sd
	drop mean sd
}
bysort year: su sat*

/* Leave the first score for duplicates. Assume earlier year of data has the earlier score if date missing from one of the duplicates. */

duplicates report studentno
bysort studentno: egen rank_year = rank(year)
bysort studentno: egen rank_date = rank(date)

gen temp = missing(date)
bysort studentno: egen miss = mean(temp)
drop temp

duplicates tag studentno, gen(dupl)
tab dupl, miss
tab rank_year rank_date if dupl == 1 & miss == 0 

drop if dupl == 1 & rank_date == 2
drop rank_* miss dupl

duplicates report studentno

/* Drop redundant variables, generate year variables (refers to fall term) */

keep studentno year satv satm satw sat satraw
foreach x in satv satm satw sat {
	gen yr_`x' = year - 1
}
drop year

/* Save data */

compress
sort studentno
save "$dir/Data/sat_data.dta", replace

/* Close log file */

log close


/****************************/
/* PREPARATION OF PSAT DATA */
/****************************/


/* Open log file */

capture: log close
log using "$dir/Results_and_Notes/psat_data.log", replace

/* Load data */

use "$dir/Data/psat_clean05212011.dta", clear
desc
compress

/* Rename variables */

rename crit_read psatr
rename math psatm
rename writ psatw
rename nmsc psat
rename fileyr year

/* Check the mathing situation */

tab level, miss

/* Check summary statistics of the score variables */

bysort year: su psat*

/* Recode total score as missing if any of the section scores missing (coded as zero in the original file) */

replace psat = . if psatr == . | psatm == . | psatw == .
su psat*

/* Drop observation if all scores missing */

drop if psatr == . & psatm == . & psatw == . & psat == .

/* Check that no duplicates of same score in same year */

duplicates report studentno psatr psatm psatw psat year

/* Keep the first observation from duplicate scores from different year */

duplicates tag studentno psatr psatm psatw psat, gen(temp)
gsort studentno year
list if temp > 0
duplicates drop studentno psatr psatm psatw psat, force
list if temp > 0
drop temp

/* Replace total score to be the sum without writing */

replace psat = psatr + psatm

/* Standardize scores within file years */

gen psatraw = psat
foreach x in psatr psatm psatw psat {
	bysort year: egen mean = mean(`x')
	bysort year: egen sd = sd(`x')
	replace `x' = (`x' - mean)/sd
	drop mean sd
}
bysort year: su psat*

/* Keep the observation with worse total score if several observations in same year */

duplicates tag studentno year, gen (temp)
gsort studentno year temp
list if temp > 0
drop temp

gsort studentno psat
duplicates drop studentno year, force

/* Keep the first one of observation from different years */

duplicates report studentno
gsort studentno year
duplicates drop studentno, force

/* Drop redundant variables and generate year variables that refer to fall term as in MCAS data */

keep studentno year psatr psatm psatw psat psatraw 
foreach x in psatr psatm psatw psat {
	gen yr_`x' = year - 1
}
drop year

/* Save data */

compress
sort studentno
save "$dir/Data/psat_data.dta", replace

/* Close log file */

log close


/***************************/
/* PREPARATION OF AP DATA */
/***************************/


/* Open log file */

capture: log close
log using "$dir/Results_and_Notes/ap_data.log", replace

/* Generate empty data */

clear
gen studentno = .
gen ap_year = .
tostring *, replace
save "$dir/Data/ap_data.dta", replace
 
/* Add 2005 data */

use "$dir/Data/AP2005.DTA", clear
desc
compress
destring *, replace

rename histusscr ap7 
rename arthistscr ap13 
rename studioartdrwngscr ap14 
rename STUDIOART2DDESIGNSCR ap15 
rename STUDIOART3DDESIGNSCR ap16 
rename bioscr ap20 
rename chemscr ap25 
gen ap28 = .
rename compsciascr ap31
rename compsciabscr ap33
rename econmicroscr ap34
rename econmacroscr ap35
rename englangcompscr ap36
rename englitcompscr ap37
rename environsciscr ap40
rename histeurscr ap43
rename frenlangscr ap48
rename frenlitscr ap51
rename humangeoscr ap53
rename germanlangscr ap55
rename govtpolusscr ap57
rename govtpolcompscr ap58
gen ap59 = .
rename latinvergilscr ap60
rename latinlit ap61
gen ap62 = .
gen ap64 = .
rename calcabscr ap66
rename calcbcscr ap68
gen ap69 = .
rename musicthry ap75
gen ap76 = .
gen ap77 = .
rename physicsbscr ap78
rename physicscmechscr ap80
rename PHYSICSCE_MSCR ap82
rename pyschologyscr ap85
rename spanishlangscr ap87
rename spanishlitscr ap89
rename statscr ap90
rename histworldscr ap93

gen ap_year = 2005
keep studentno ap*
tostring *, replace
append using "$dir/Data/ap_data.dta"
save "$dir/Data/ap_data.dta", replace

/* Add 2006 data */

use "$dir/Data/AP2006.DTA", clear
desc
compress
destring *, replace

forval i = 1/30 {
	local j = "`i'"
	if `i' < 10 {
		local j = "0`i'"
	}
	rename  AP`j'Code code`i'
	rename  AP`j'AdminYr year`i'
	rename  AP`j'ScrGrd score`i'
}
rename StudentNo studentno
keep studentno code* year* score*
reshape long code year score, i(studentno) j(number)
drop if code == .
duplicates tag studentno code, gen(temp)
gsort studentno code year score
list if temp > 0
duplicates drop studentno code, force
list if temp > 0
drop temp

levelsof code, local(codes)
foreach i of local codes {
	bysort studentno: egen temp = mean(score) if code == `i'
	bysort studentno: egen ap`i' = mean(temp)
	drop temp
}
duplicates drop studentno, force

gen ap_year = 2006
keep studentno ap*
tostring *, replace
append using "$dir/Data/ap_data.dta"
save "$dir/Data/ap_data.dta", replace

/* Add 2007 data */

use "$dir/Data/AP2007.DTA", clear
desc
compress
destring *, replace

rename ushistgr ap7 
rename arthisgr ap13 
rename artstdgr ap14 
rename ARTST2GR ap15 
rename ART3DGR ap16 
rename biolgr ap20 
rename chemgr ap25 
rename chinesgr ap28 
rename comscagr ap31
rename comscbgr ap33
rename econmigr ap34
rename econmagr ap35
rename englangr ap36
rename englitgr ap37
rename envscigr ap40
rename eurhisgr ap43
rename frnlangr ap48
rename frenltgr ap51
rename humgeogr ap53
rename gerlagr ap55
rename govusgr ap57
rename govcomgr ap58
gen ap59 = .
rename latinvgr ap60
rename latincgr ap61
rename italgr ap62
rename japangr ap64
rename calcabgr ap66
rename calcbcgr ap68
gen ap69 = .
rename musictgr ap75
gen ap76 = .
gen ap77 = .
rename physbgr ap78
rename physmgr ap80
rename physemgr ap82
rename psychgr ap85
rename spanlagr ap87
rename spanltgr ap89
rename statgr ap90
rename wdhistgr ap93

gen ap_year = 2007
rename StudentNo studentno
keep studentno ap*
tostring *, replace
append using "$dir/Data/ap_data.dta"
save "$dir/Data/ap_data.dta", replace

/* Add 2008 data */

use "$dir/Data/AP2008.DTA", clear
desc
compress
destring *, replace

forval i = 1/30 {
	local j = "`i'"
	if `i' < 10 {
		local j = "0`i'"
	}
	rename  AP`j'Code code`i'
	rename  AP`j'AdminYr year`i'
	rename  AP`j'ScrGrd score`i'
}
rename StudentNo studentno
keep studentno code* year* score*
reshape long code year score, i(studentno) j(number)
drop if code == .
duplicates tag studentno code, gen(temp)
gsort studentno code year score
list if temp > 0
duplicates drop studentno code, force
list if temp > 0
drop temp

levelsof code, local(codes)
foreach i of local codes {
	bysort studentno: egen temp = mean(score) if code == `i'
	bysort studentno: egen ap`i' = mean(temp)
	drop temp
}
duplicates drop studentno, force

gen ap_year = 2008
keep studentno ap*
tostring *, replace
append using "$dir/Data/ap_data.dta"
save "$dir/Data/ap_data.dta", replace

/* Add 2009 data */

use "$dir/Data/AP2009.DTA", clear
desc
compress
destring *, replace

forval i = 1/30 {
	local j = "`i'"
	if `i' < 10 {
		local j = "0`i'"
	}
	rename  AP`j'Code code`i'
	rename  AP`j'AdminYr year`i'
	rename  AP`j'ScrGrd score`i'
}
rename StudentNo studentno
keep studentno code* year* score*
reshape long code year score, i(studentno) j(number)
drop if code == .
duplicates tag studentno code, gen(temp)
gsort studentno code year score
list if temp > 0
duplicates drop studentno code, force
list if temp > 0
drop temp

levelsof code, local(codes)
foreach i of local codes {
	bysort studentno: egen temp = mean(score) if code == `i'
	bysort studentno: egen ap`i' = mean(temp)
	drop temp
}
duplicates drop studentno, force

gen ap_year = 2009
keep studentno ap*
tostring *, replace
append using "$dir/Data/ap_data.dta"
save "$dir/Data/ap_data.dta", replace

/* Add 2010 data */

use "$dir/Data/AP2010.DTA", clear
desc
compress
destring *, replace

forval i = 1/30 {
	local j = "`i'"
	if `i' < 10 {
		local j = "0`i'"
	}
	rename  AP`j'Code code`i'
	rename  AP`j'AdminYr year`i'
	rename  AP`j'ScrGrd score`i'
}
rename StudentNo studentno
keep studentno code* year* score*
reshape long code year score, i(studentno) j(number)
drop if code == .
duplicates tag studentno code, gen(temp)
gsort studentno code year score
list if temp > 0
duplicates drop studentno code, force
list if temp > 0
drop temp

levelsof code, local(codes)
foreach i of local codes {
	bysort studentno: egen temp = mean(score) if code == `i'
	bysort studentno: egen ap`i' = mean(temp)
	drop temp
}
duplicates drop studentno, force

gen ap_year = 2010
keep studentno ap*
tostring *, replace
append using "$dir/Data/ap_data.dta"
save "$dir/Data/ap_data.dta", replace

/* Get rid of potential erroneous observations */

destring *, replace
drop if missing(studentno)

/* Check if duplicates within file years and in general  */

duplicates report studentno
duplicates report studentno ap_year

/* Generate the number of AP classes taken in each year */

levelsof ap_year, local(years)
foreach yr in `years' {
	egen temp = rownonmiss(ap*) if ap_year == `yr'
	replace temp = temp - 1  if ap_year == `yr'
	bysort studentno: egen naps_`yr' = mean(temp)
	drop temp
}

/* Keep only the first record of each AP class and drop duplicates */

foreach x of varlist ap* {
	if "`x'" != "ap_year" {
		bysort studentno: egen min_year = min(ap_year) if `x' != .
		replace `x' = . if ap_year != min_year
		bysort studentno: egen temp = mean(`x')
		replace `x' = temp
		drop min_year temp
	}
}
drop ap_year
duplicates drop studentno, force

/* Drop Calculus AP subscore if a kid has also regular Calculus AB score */

replace ap69 = . if ap66 != .

/* Rename the AP course variables */

foreach var of varlist ap* {
	rename `var' course_`var'
}

/* Generate the number of grades 4 and 5 */

egen apnbrgrade4 = anycount(course_ap*), value(4)
egen apnbrgrade5 = anycount(course_ap*), value(5)

/* Generate total number of classes & number of credits among all and selected classes */

egen apnbrall = rownonmiss(course_ap*)
egen apscrall = rowtotal(course_ap*)

foreach x in 13 14 15 16 75 76 77 {
	rename course_ap`x' temp`x'
}
egen apnbrall2 = rownonmiss(course_ap*)
egen apscrall2 = rowtotal(course_ap*)
foreach x in 13 14 15 16 75 76 77 {
	rename temp`x' course_ap`x'
}

local ap500 = "course_ap7 course_ap20 course_ap25 course_ap34 course_ap35 course_ap36 course_ap37 course_ap43 course_ap57 course_ap66 course_ap68 course_ap78 course_ap80 course_ap82 course_ap90"
egen apnbr500 = rownonmiss(`ap500')
egen apscr500 = rowtotal(`ap500')

local ap1000 = "course_ap7 course_ap20 course_ap36 course_ap37 course_ap57 course_ap66"
egen apnbr1000 = rownonmiss(`ap1000')
egen apscr1000 = rowtotal(`ap1000')

/* Generate dummy for having taken one of the stem classes */

local apstem = "course_ap20 course_ap25 course_ap31 course_ap33 course_ap40 course_ap66 course_ap68 course_ap78 course_ap80 course_ap82 course_ap90"
egen temp = rownonmiss(`apstem')
gen take_apstem = (temp >= 0) if temp != .

/* Double-check that there are no duplicates and keep only relevant variables */

duplicates report studentno
keep studentno apnbr* apscr* naps_* take_apstem

/* Save data */

compress
sort studentno
save "$dir/Data/ap_data.dta", replace

/* Close log */

log close


/********************************/
/* PREPARATION OF BARRON'S DATA */
/********************************/


/* Open log file */

capture: log close
log using "$dir/Results_and_Notes/barrons_data.log", replace

/* Open data */

use "$dir/Data/barrons2009.dta", clear
desc
compress

/* Chech the college identifier */

destring opeid, replace
tab opeid, miss
rename opeid college_code

/* Drop redundant variables */

keep college_code instnm barrons_rank_2009

/* Save data */

compress
sort college_code
save "$dir/Data/barrons_data.dta", replace

/* Close log */

log close


/***************************/
/* PREPARATION OF NSC DATA */
/***************************/


/* Open log file */

capture: log close
log using "$dir/Results_and_Notes/nsc_data.log", replace

/* Open data */

use "$dir/Data/boston_nsc.dta", clear
desc
compress

/* Modify studentno */

replace studentno = plural(2, studentno, "-_")
destring studentno, replace

/* Modify the college code */

tab college_code, miss
gen temp = substr(college_code, -3, .)
tab temp
replace temp = substr(temp, 2, .)
replace college_code = substr(college_code, 1, 6) + temp
destring college_code, replace
tab college_code, miss

/* Add information on Barron's ranking */

sort college_code
merge college_code using "$dir/Data/barrons_data.dta"
tab _merge
drop if _merge == 2
drop _merge

/* Generate indicators for Barron's rankings */

tab barrons_rank_2009, miss

gen barrons_MC = (barrons_rank_2009 == "Most Competitive")
gen barrons_HC = (barrons_MC == 1 | barrons_rank_2009 == "Highly Competitive")
gen barrons_VC = (barrons_HC == 1 | barrons_rank_2009 == "Very Competitive")
gen barrons_C = (barrons_VC == 1 | barrons_rank_2009 == "Competitive")
gen barrons_LC = (barrons_C == 1 | barrons_rank_2009 == "Less Competitive")
gen barrons_S = (barrons_LC == 1 | barrons_rank_2009 == "Special")

/* Generate variables for semesters enrolled in 2 and 4 year colleges */

tab years24, miss
gen temp2 = ((years24 == "2" | years24 == "L") & enrollment_begin != .)
gen temp4 = (years24 == "4" & enrollment_begin != .)

bysort studentno: egen semesters_2yr = total(temp2)
bysort studentno: egen semesters_4yr = total(temp4)

drop temp2 temp4

/* Generate dummies for attending any 2 or 4 year colleges */

gen attend_2yr = (semesters_2yr > 0 & semesters_2yr != .)
gen attend_4yr = (semesters_4yr > 0 & semesters_4yr != .)
gen attend_any = (attend_2yr == 1 | attend_4yr == 1)

/* Generate dummies for attending any 2 or 4 year colleges for at least 4 semesters */

gen persist_2yr = (semesters_2yr >= 4 & semesters_2yr != .)
gen persist_4yr = (semesters_4yr >= 4 & semesters_4yr != .)
gen persist_any = (persist_2yr == 1 | persist_4yr == 1)

/* Generate dummies for graduating from any 2 or 4 year colleges */

gen temp2 = ((years24 == "2" | years24 == "L") &  graduation_date != .)
gen temp4 = (years24 == "4" & graduation_date != .)

bysort studentno: egen graduate_2yr = total(temp2)
replace graduate_2yr = 1 if graduate_2yr > 0 & graduate_2yr != .
bysort studentno: egen graduate_4yr = total(temp4)
replace graduate_4yr = 1 if graduate_4yr > 0 & graduate_4yr != .
gen graduate_any = (graduate_2yr == 1 | graduate_4yr == 1)

drop temp2 temp4

/* Change the values of attending and persisting from 0 to 1 if the student graduated */

foreach i in 2yr 4yr any {
	replace attend_`i' = 1 if graduate_`i' == 1
	replace persist_`i' = 1 if graduate_`i' == 1
}

/* Generate variable for attending in a IVY league school */

gen attend_ivy = 0
replace attend_ivy = 1 if college_name == "BROWN UNIVERSITY"
replace attend_ivy = 1 if college_name == "COLUMBIA UNIVERSITY"
replace attend_ivy = 1 if college_name == "CORNELL UNIVERSITY"
replace attend_ivy = 1 if college_name == "CORNELL UNIVERSITY - ENG/ARCH/IRL/HUM.E"
replace attend_ivy = 1 if college_name == "CORNELL UNIVERSITY-ARTS"
replace attend_ivy = 1 if college_name == "DARTMOUTH COLLEGE"
replace attend_ivy = 1 if college_name == "HARVARD - KENNEDY SCHOOL OF GOVERNMENT"
replace attend_ivy = 1 if college_name == "HARVARD UNIVERSITY"
replace attend_ivy = 1 if college_name == "HARVARD UNIVERSITY - CONTINUING ED"
replace attend_ivy = 1 if college_name == "PRINCETON UNIVERSITY"
replace attend_ivy = 1 if college_name == "UNIVERSITY OF PENNSYLVANIA"
replace attend_ivy = 1 if college_name == "YALE UNIVERSITY"

/* Keep the first observation of each student according to the enrollment date (or graduation date if that is the only information) */

gsort studentno enrollment_begin graduation_date
duplicates drop studentno, force

/* Drop redundant variables */

drop barrons_rank_2009
keep studentno attend_* persist_* graduate_* barrons_*

/* Check how many BPS students going to selective colleges are from the exam schools */

save "$dir/Data/nsc_data.dta", replace

sort studentno
merge studentno using "$dir/Data/12th_graders_enrollment_data.dta"
tab _merge, miss

tab schyear if attend_ivy == 1
tab schyear if attend_ivy == 1 & (school == 1010 | school == 1020 | school == 1030)

tab schyear if attend_ivy == 1 & (black == 1 | hisp == 1)
tab schyear if attend_ivy == 1 & (black == 1 | hisp == 1) & (school == 1010 | school == 1020 | school == 1030)

tab schyear if attend_ivy == 1 & free == 1
tab schyear if attend_ivy == 1 & free == 1 & (school == 1010 | school == 1020 | school == 1030)

tab schyear if barrons_MC == 1
tab schyear if barrons_MC == 1 & (school == 1010 | school == 1020 | school == 1030)

tab schyear if barrons_MC == 1 & (black == 1 | hisp == 1)
tab schyear if barrons_MC == 1 & (black == 1 | hisp == 1) & (school == 1010 | school == 1020 | school == 1030)

tab schyear if barrons_MC == 1 & free == 1
tab schyear if barrons_MC == 1 & free == 1 & (school == 1010 | school == 1020 | school == 1030)

tab schyear if (barrons_MC == 1 | barrons_HC == 1)
tab schyear if (barrons_MC == 1 | barrons_HC == 1) & (school == 1010 | school == 1020 | school == 1030)

tab schyear if (barrons_MC == 1 | barrons_HC == 1) & (black == 1 | hisp == 1)
tab schyear if (barrons_MC == 1 | barrons_HC == 1) & (black == 1 | hisp == 1) & (school == 1010 | school == 1020 | school == 1030)

tab schyear if (barrons_MC == 1 | barrons_HC == 1) & free == 1
tab schyear if (barrons_MC == 1 | barrons_HC == 1) & free == 1 & (school == 1010 | school == 1020 | school == 1030)

use "$dir/Data/nsc_data.dta", clear

/* Save data */

compress
sort studentno
save "$dir/Data/nsc_data.dta", replace

/* Close log */

log close


/************************************************/
/* PREPARE THE ADDRESS/DISTANCE FILES FROM ALEX */
/************************************************/

/* Prepare the files from Alex */

use "$dir/Data/studentID.dta", clear
gsort schyear studentno
save "$dir/Data/studentID.dta", replace

use "$dir/Data/schaddyears.dta", clear
rename year schyear
rename school schoolid
save "$dir/Data/schadd2.dta", replace

use "$dir/Data/eucDistAdd.dta", clear
drop if addid==. | schaddid==.
tostring addid schaddid, replace force
save "$dir/Data/eucDistAddString.dta", replace


/**************************/
/* PREPARATION OF ID DATA */
/**************************/


/* Open log file */

capture: log close
log using "$dir/Results_and_Notes/id_data.log", replace

/* Load data */

use "$dir/Data/exam_app_idlist.dta", clear
desc
compress
count

/* Destring variables */

replace reg_num = "" if reg_num == "NULL"
destring *, replace

/* Construct date of birth variable */

gen dob_num = string(dob)
replace dob_num = "0" + dob_num if length(dob_num) == 5
drop dob

gen mn = real(substr(dob_num), 1, 2)
tab mn, miss

gen dy = real(substr(dob_num), 3, 2)
tab dy, miss

gen yr = 1900 + real(substr(dob_num), 5, 2)
tab yr, miss

gen dob_id = mdy(mn, dy, yr)
format dob_id %d
drop mn dy yr dob_num

/* Check whether we have multiple observations of a registration number within a given year  */

duplicates report schyear reg_num

/* Check missing registration numbers */

gen temp = missing(reg_num)
tab schyear temp, miss
drop temp

/* Save id data */

compress
gsort schyear reg_num 
save "$dir/Data/id_data.dta", replace

/* Close log file */

log close


/*******************************************************************/
/* MERGING OF TEST SCORE, APPLICATION, ENROLLMENT AND ADDRESS DATA */
/*******************************************************************/


/* Open log file */

capture: log close
log using "$dir/Results_and_Notes/merging.log", replace

/* Load enrollment data */

use "$dir/Data/enrollment_data.dta", clear

tab schyear if studentno != .
tab schyear if studentno == .

/* Check whether we have multiple observations of a student number within a given year  */

duplicates report schyear studentno

/* Add MCAS data */

sort studentno
merge studentno using "$dir/Data/test_score_data.dta"

tab schyear _merge, miss
tab schyear _merge if grade == 6, miss
tab schyear _merge if grade == 8, miss

drop if _merge == 2
drop _merge

/* Add SAT data */

sort studentno
merge studentno using "$dir/Data/sat_data.dta"

tab _merge, miss
tab schyear _merge if grade == 6, miss
tab schyear _merge if grade == 8, miss

drop if _merge == 2
drop _merge

/* Add PSAT data */

sort studentno
merge studentno using "$dir/Data/psat_data.dta"

tab _merge, miss
tab schyear _merge if grade == 6, miss
tab schyear _merge if grade == 8, miss

drop if _merge == 2
drop _merge

/* Add AP data */

sort studentno
merge studentno using "$dir/Data/ap_data.dta"

tab _merge, miss
tab schyear _merge if grade == 6, miss
tab schyear _merge if grade == 8, miss

drop if _merge == 2
drop _merge

/* Add NSC data */

sort studentno
merge studentno using "$dir/Data/nsc_data.dta"

tab _merge, miss
tab schyear _merge if grade == 6, miss
tab schyear _merge if grade == 8, miss

drop if _merge == 2
drop _merge

/* Add address ID data for later distance merge */

gsort schyear studentno
merge schyear studentno using "$dir/Data/studentID.dta"

tab schyear _merge
keep if _merge == 1 | _merge == 3
drop _merge demo_grade

/* Save a temporary version of the merged data data */

gsort schyear studentno
save "$dir/Data/merged_data.dta", replace

/* Load application data */

use "$dir/Data/application_data.dta", clear

/* Document the number of applicants before merging */

tab schyear if app_grade == 7, miss matcell(temp)
mat kuv_7 = temp
tab schyear if app_grade == 7 & offer_yn == 0, miss matcell(temp)
mat kuv_7_out = temp
tab schyear if app_grade == 7 & offer_yn == 1, miss matcell(temp)
mat kuv_7_in = temp

tab schyear if app_grade == 9, miss matcell(temp)
mat kuv_9 = temp
tab schyear if app_grade == 9 & offer_yn == 0, miss matcell(temp)
mat kuv_9_out = temp
tab schyear if app_grade == 9 & offer_yn == 1, miss matcell(temp)
mat kuv_9_in = temp

tab schyear, miss matcell(temp)
mat kuv_all = temp
tab schyear if offer_yn == 0, miss matcell(temp)
mat kuv_all_out = temp
tab schyear if offer_yn == 1, miss matcell(temp)
mat kuv_all_in = temp

global columns = "ALL"

mat rownames kuv_7 = 1995 1996 1997 1998 1999 2000 2001 2002 2003 2004 2005 2006 2007 2008 2009
mat rownames kuv_7_out = 1995 1996 1997 1998 1999 2000 2001 2002 2003 2004 2005 2006 2007 2008 2009
mat rownames kuv_7_in = 1995 1996 1997 1998 1999 2000 2001 2002 2003 2004 2005 2006 2007 2008 2009

mat rownames kuv_9 = 1995 1996 1997 1998 1999 2000 2001 2002 2003 2004 2005 2006 2007 2008 2009
mat rownames kuv_9_out = 1995 1996 1997 1998 1999 2000 2001 2002 2003 2004 2005 2006 2007 2008 2009
mat rownames kuv_9_in = 1995 1996 1997 1998 1999 2000 2001 2002 2003 2004 2005 2006 2007 2008 2009

mat rownames kuv_all = 1995 1996 1997 1998 1999 2000 2001 2002 2003 2004 2005 2006 2007 2008 2009
mat rownames kuv_all_out = 1995 1996 1997 1998 1999 2000 2001 2002 2003 2004 2005 2006 2007 2008 2009
mat rownames kuv_all_in = 1995 1996 1997 1998 1999 2000 2001 2002 2003 2004 2005 2006 2007 2008 2009

/* Check whether we have multiple observations of a registration number within a given year */

duplicates tag schyear reg_num, gen(temp)
tab temp, miss
drop temp

/* Check whether we have multiple observations of a student number within a given year  */

duplicates tag schyear studentno, gen(temp)
tab temp, miss
drop temp

/* Add duplicate ids ### NULL VALUES IN REG_NUM IN 1995-2000 IN ABOUT 800 CASES ### */

gsort schyear reg_num 
merge schyear reg_num using "$dir/Data/id_data.dta"

tab _merge, miss
drop if _merge == 2
drop _merge

replace id1 = . if reg_num == .
replace id2 = . if reg_num == .

/* In some cases id1 and id2 equal reg_num and studentno or at least one of these is true. 
I ignore this and handle the potential duplicate matches resulting from this when handling 
all the other duplicate matches. */

tab schyear if reg_num == id1
tab schyear if reg_num == id1 & studentno == id2
tab schyear if reg_num == id1 & studentno != id2

tab schyear if reg_num == id2
tab schyear if reg_num == id2 & studentno == id1
tab schyear if reg_num == id2 & studentno != id1

/* Check whether student number is always equal to one or the other of the alternative ids */

count if studentno != id1 & studentno != id2 & id1 != . & id2 != .

/* Check how many multiple observations of a registration number we have now */

duplicates report schyear reg_num

/* Check whether we have multiple observations of a student number within a given year  */

duplicates report schyear studentno

/* Save a version of the data with all the ids */

gsort schyear studentno
save "$dir/Data/application_data_with_multiple_ids.dta", replace

/* Merge students based on studentno in the application file */

use "$dir/Data/application_data_with_multiple_ids.dta", replace

gsort schyear studentno
merge schyear studentno using "$dir/Data/merged_data.dta"

tab schyear _merge, miss
drop if _merge != 3
drop _merge

/* Save a temporary version of this data */

gen merge_file = 1
gsort schyear studentno
save "$dir/Data/temporary_data1.dta", replace

/* Merging students with multiple ids based on the first alternative */

use  "$dir/Data/application_data_with_multiple_ids.dta", replace
keep if id1 != .
replace studentno = id1

gsort schyear studentno
merge schyear studentno using "$dir/Data/merged_data.dta"

tab schyear _merge, miss
drop if _merge != 3
drop _merge

/* Save a temporary version of this data*/

gen merge_file = 2
gsort schyear studentno
save "$dir/Data/temporary_data2.dta", replace

/* Merging students with multiple ids based on the second alternative */

use  "$dir/Data/application_data_with_multiple_ids.dta", replace
keep if id2 != .
replace studentno = id2

gsort schyear studentno
merge schyear studentno using "$dir/Data/merged_data.dta"

tab schyear _merge, miss
drop if _merge != 3
drop _merge

/* Save a temporary version of this data*/

gen merge_file = 3
gsort schyear studentno
save "$dir/Data/temporary_data3.dta", replace

/* Stack the temporay merged files into one */

use "$dir/Data/temporary_data1.dta", clear
count
append using "$dir/Data/temporary_data2.dta"
count
append using "$dir/Data/temporary_data3.dta"
count

tab schyear app_grade, miss

/* Remove duplicate observations based on year, student number and registration number. 
These should be duplicates of the same student since registration number is a unique identifier 
in the application file (not anymore true but should be handled with the code below) and 
matching was done based on year and student number. I keep one of the duplicates for cases that have
the same values for various variables and drop the rest. */

duplicates tag schyear studentno reg_num, gen(temp)
tab temp, miss
drop if (dob != dob_id & dob != . & dob_id != . & temp > 0)
drop temp

duplicates tag schyear studentno reg_num, gen(temp)
tab temp, miss
quietly: count if temp > 0
if r(N) > 0 {
	duplicates tag schyear studentno reg_num grade school* verb quan read *rank eng* math* if temp > 0, gen(temp2)
	tab temp2, miss
}
drop if temp2 == 0
duplicates drop schyear studentno reg_num, force
drop temp temp2

/* Duplicate guys with the same year and registration number but different student numbers are 
problematic. One of these observations has (potentially) the wrong matched variables. I will 
handle this by keeping only those of the duplicate observations with matching date of births in 
the id and enrollment data. After that I will drop all the rest of these duplicate observations 
since we cannot know which one is the right match. However, I'll leave one of the duplicates in 
cases where they seem to be the same student given a bunch of application and enrollment file variables. */

duplicates tag schyear reg_num, gen(temp)
tab temp, miss
drop if (dob != dob_id & dob != . & dob_id != . & temp > 0)
drop temp

duplicates tag schyear reg_num, gen(temp)
tab temp, miss
quietly: count if temp > 0
if r(N) >0 {
	duplicates tag schyear reg_num grade school* verb quan read *rank eng* math* if temp > 0, gen(temp2)
	tab temp2, miss
}
drop if temp2 == 0
duplicates drop schyear reg_num, force
drop temp temp2

/* Handle duplicate student numbers within a year with a similar strategy. That is, keep a duplicate with 
a matching date of birth and remove the rest but check also if seem to be the same student given a bunch of 
application and enrollment file variables. */

duplicates tag schyear studentno, gen(temp)
tab temp, miss
drop if (dob != dob_id & dob != . & dob_id != . & temp > 0)
drop temp

duplicates tag schyear studentno, gen(temp)
tab temp, miss
quietly: count if temp > 0
if r(N) >0 {
	duplicates tag schyear studentno grade school* verb quan read *rank eng* math* if temp > 0, gen(temp2)
	tab temp2, miss
}
drop if temp2 == 0
duplicates drop schyear studentno, force
drop temp temp2

/* Handle rest of the matches that seem to be incorrect even though there is no more duplicates (grade, date of birth, sex) */

tab schyear app_grade if app_grade != grade + 1
drop if app_grade != grade + 1

tab schyear app_grade if (dob != dob_id & dob != . & dob_id != .), miss
drop if dob != dob_id & dob != . & dob_id != .

tab schyear app_grade if (female_a != female & female_a != . & female != .), miss
drop if female_a != female & female_a != . & female != .

/* Check the age of the applicants in the beginning of the application year */

gen age = (mdy(1, 1, schyear) - dob)/365.25
bysort app_grade: su age, d

/* Document the final number of matched applicants */

tab schyear if app_grade == 7, miss matcell(temp)
mat kuv_7 = (kuv_7, (0\ temp))
tab schyear if app_grade == 7 & offer_yn == 0, miss matcell(temp)
mat kuv_7_out = (kuv_7_out, (0\ temp))
tab schyear if app_grade == 7 & offer_yn == 1, miss matcell(temp)
mat kuv_7_in = (kuv_7_in, (0\ temp))

tab schyear if app_grade == 9, miss matcell(temp)
mat kuv_9 = (kuv_9, (0\ temp))
tab schyear if app_grade == 9 & offer_yn == 0, miss matcell(temp)
mat kuv_9_out = (kuv_9_out, (0\ temp))
tab schyear if app_grade == 9 & offer_yn == 1, miss matcell(temp)
mat kuv_9_in = (kuv_9_in, (0\ temp))

tab schyear, miss matcell(temp)
mat kuv_all = (kuv_all, (0\ temp))
tab schyear if offer_yn == 0, miss matcell(temp)
mat kuv_all_out = (kuv_all_out, (0\ temp))
tab schyear if offer_yn == 1, miss matcell(temp)
mat kuv_all_in = (kuv_all_in, (0\ temp))

global columns = "$columns" + " MATCH"

/* Document the number of observations after some additional restrictions */

preserve

keep if yrs_in_exam_cur == 0 | yrs_in_exam_cur == .

tab schyear if app_grade == 7, miss matcell(temp)
mat kuv_7 = (kuv_7, (0\ temp))
tab schyear if app_grade == 9, miss matcell(temp)
mat kuv_9 = (kuv_9, (0\ temp))
tab schyear, miss matcell(temp)
mat kuv_all = (kuv_all, (0\ temp))
global columns = "$columns" + " NEVER"

keep if (app_grade == 7 & (eng7 != . | math7 !=. | eng8 != . | math8 != . | eng10 != . | math10 != .)) | (app_grade == 9 & (eng10 != . | math10 != .))

tab schyear if app_grade == 7, miss matcell(temp)
mat kuv_7 = (kuv_7, (0\ temp\ 0))
tab schyear if app_grade == 9, miss matcell(temp)
mat kuv_9 = (kuv_9, (0\ 0\ 0\ 0\ temp\ 0\ 0))
tab schyear, miss matcell(temp)
mat kuv_all = (kuv_all, (0\ temp\ 0))
global columns = "$columns" + " OUTCOME"

restore

/* Produce the numbers for the table of match success */

local years = `""1999".."2008""'
mat a1 = diag((kuv_7[`years', 1]))
mat a1 = diag((vecdiag(a1), trace(a1)))
mat b1 = diag((kuv_7[`years', 2]))
mat b1 = diag((vecdiag(b1), trace(b1)))
mat a2 = diag((kuv_7_in[`years', 1]))
mat a2 = diag((vecdiag(a2), trace(a2)))
mat b2 = diag((kuv_7_in[`years', 2]))
mat b2 = diag((vecdiag(b2), trace(b2)))
mat a3 = diag((kuv_7_out[`years', 1]))
mat a3 = diag((vecdiag(a3), trace(a3)))
mat b3 = diag((kuv_7_out[`years', 2]))
mat b3 = diag((vecdiag(b3), trace(b3)))

mat res = (vecdiag(a1)', vecdiag(invsym(a1)*b1)', vecdiag(invsym(a2)*b2)', vecdiag(invsym(a3)*b3)')
mat list res

local years = `""2001".."2007""'
mat a1 = diag((kuv_9[`years', 1]))
mat a1 = diag((vecdiag(a1), trace(a1)))
mat b1 = diag((kuv_9[`years', 2]))
mat b1 = diag((vecdiag(b1), trace(b1)))
mat a2 = diag((kuv_9_in[`years', 1]))
mat a2 = diag((vecdiag(a2), trace(a2)))
mat b2 = diag((kuv_9_in[`years', 2]))
mat b2 = diag((vecdiag(b2), trace(b2)))
mat a3 = diag((kuv_9_out[`years', 1]))
mat a3 = diag((vecdiag(a3), trace(a3)))
mat b3 = diag((kuv_9_out[`years', 2]))
mat b3 = diag((vecdiag(b3), trace(b3)))

mat res = (vecdiag(a1)', vecdiag(invsym(a1)*b1)', vecdiag(invsym(a2)*b2)', vecdiag(invsym(a3)*b3)')
mat list res

/* Produce the attrition summary table */

preserve

keep if yrs_in_exam_cur == 0 | yrs_in_exam_cur == .

egen count_math = rownonmiss(math7 math8 math10)
egen count_eng = rownonmiss(eng7 eng8 eng10)
gen observations  = 1
gen has_scores = (count_math > 0 | count_eng > 0)
tabstat observations has_scores count_math count_eng if app_grade == 7, by(schyear) stat(sum)
drop observations has_scores count_math count_eng 

egen count_math = rownonmiss(math10)
egen count_eng = rownonmiss(eng10)
gen observations  = 1
gen has_scores = (count_math > 0 | count_eng > 0)
tabstat observations has_scores count_math count_eng if app_grade == 9, by(schyear) stat(sum)
drop observations has_scores count_math count_eng 

egen count_psat = rownonmiss(psat)
egen count_sat = rownonmiss(sat)
egen count_ap = rownonmiss(apscrall)
egen count_nsc = rownonmiss(attend_4yr)
gen observations = 1
tabstat observations count_psat count_sat count_ap count_nsc if app_grade == 7, by(schyear) stat(sum)
tabstat observations count_psat count_sat count_ap count_nsc if app_grade == 9, by(schyear) stat(sum)
drop observations count_psat count_sat count_ap count_nsc

restore

/* Make a data of all the public school applicants who we were not able to merge */

save "$dir/Data/temporary_data.dta", replace

keep merge_check_id
sort merge_check_id
save "$dir/Data/merge_check_data.dta", replace

use "$dir/Data/application_data.dta", clear
sort merge_check_id
merge merge_check_id using "$dir/Data/merge_check_data.dta"
tab schyear _merge, miss

gen merge_succesfull = (_merge == 3)
drop _merge

save "$dir/Data/merge_check_data.dta", replace

use "$dir/Data/temporary_data.dta", replace

/* Add to this data all the observations from the merged enrollment-test score data */

append using "$dir/Data/merged_data.dta"
tab schyear app_grade, miss

/* Save the final merged data*/

compress
gsort schyear studentno
save "$dir/Data/merged_data.dta", replace

/* Close log file */

log close


/************************************/
/* PREPARE MERGED DATA FOR ANALYSIS */
/************************************/


/* Open log file */

capture: log close
log using "$dir/Results_and_Notes/prepare_analysis.log", replace

/* Load data */

use "$dir/Data/merged_data.dta", clear
desc

/* Generate next_school and next_grade variable */

gen next_school = .
gen next_grade = .
forval yr = 1996/2008 {
	local apu = `yr' + 1	
	replace next_school = school_`apu' if schyear == `yr'
	replace next_grade = grade_`apu' if schyear == `yr'
}

/* Generate exam_school variable */

gen exam_school = ""
replace exam_school = "BLS" if next_school == 1010
replace exam_school = "BLA" if next_school == 1020
replace exam_school = "OBR" if next_school == 1030
replace exam_school = "BPS" if (next_school != . & next_school != 1010 & next_school != 1020 & next_school != 1030) 

/* Generate variables for enrolling in exam schools (and BPS) next fall */

gen to_exam = (exam_school == "OBR" | exam_school == "BLA" | exam_school == "BLS") if exam_school != ""
gen to_obr = (exam_school == "OBR") if exam_school != ""
gen to_bla = (exam_school == "BLA") if exam_school != ""
gen to_bls = (exam_school == "BLS") if exam_school != ""
gen to_bps = (exam_school == "BPS") if exam_school != ""

/* Generate variable for being in an exam school x years after application */

forval i = 1/6 {
	gen sch`i' = .
	forval yr = 1996/2010 {
		local j = `yr' - `i'
		replace sch`i' = school_`yr' if schyear == `j'
	}
	gen inexam`i' = (sch`i' == 1010 | sch`i' == 1020 | sch`i' == 1030)
	drop sch`i'
}

replace inexam1 = . if schyear >= 2010
replace inexam2 = . if schyear >= 2009
replace inexam3 = . if schyear >= 2008
replace inexam4 = . if schyear >= 2007
replace inexam5 = . if schyear >= 2006
replace inexam6 = . if schyear >= 2005
replace inexam5 = . if app_grade == 9
replace inexam6 = . if app_grade == 9

/* Retention in school after application and in 9th grade school */

forval gr = 7/12 {
	gen sch`gr' = .
	forval i = 1996/2010 {
			replace sch`gr' = school_`i' if grade_`i' == `gr'
	}
	gen innext`gr' = (sch`gr' == next_school) if next_school != .
	replace innext`gr' = . if (grade == 6 & schyear >= 2005) | (grade == 8 & schyear >= 2007) | next_school == .
}

forval gr = 10/12 {
	gen in9sch`gr' = (sch`gr' == sch9) if sch9 != .
}

drop sch7 sch8 sch9 sch10 sch11 sch12

/* Add names for next schools */

sort next_school
merge next_school using "$dir/Data/next_school_key.dta"
tab schyear _merge, miss
drop if _merge == 2
drop _merge

replace next_school_name = string(next_school) if missing(next_school_name) == 1

/* Add teacher information for the school a kid ends up in */

save "$dir/Data/analysis_data.dta", replace

use "$dir/Data/mass doe teacher and enrollment data.dta", clear
desc
keep if substr(school, 1, 9) == "Boston - "
destring orgcode, replace
drop school
sort orgcode
merge orgcode using "$dir/Data/school_code_key.dta"
keep school year p_licensed n_core p_core_qualified st_ratio age_* total_fte

gen temp = real(substr(st_ratio, 1, 4))
drop st_ratio
rename temp st_ratio
destring *, replace

rename school next_school
rename year schyear
gsort schyear next_school
drop if next_school == .
save "$dir/Data/temporary_data.dta", replace

use "$dir/Data/analysis_data.dta", clear
gsort schyear next_school 
merge schyear next_school using "$dir/Data/temporary_data.dta"
tab schyear _merge, miss
drop if _merge == 2
drop _merge

/* Add peer characteristics in the school a kid ends up in (compute leave-one-out mean) */

gsort schyear app_grade next_school
merge schyear app_grade next_school using "$dir/Data/student_characteristics_data.dta"
tab schyear _merge, miss
drop if _merge == 2
drop _merge

gen blackhisp = (black == 1 | hisp == 1) if black !=. & hisp != .

foreach x in blackhisp free lep special female {
	replace prop_`x' = (1/(prop_`x'_N - 1))*(prop_`x'_N*prop_`x' - `x') if `x' != .
}
drop prop_*_N

/* Add peer means in the school a kid ends up in (compute leave-one-out mean) */

gsort schyear app_grade next_school
merge schyear app_grade next_school using "$dir/Data/peer_mean_data.dta"
tab schyear _merge, miss
drop if _merge == 2
drop _merge

/* Add variable for distance to next school */

gen schoolid = next_school
gsort schyear schoolid
merge m:1 schyear schoolid using "$dir/Data/schadd2.dta"
tab schyear _merge
keep if _merge == 1 | _merge == 3
drop _merge schoolid

tostring addid schaddid, replace force
merge m:1 addid schaddid using "$dir/Data/eucDistAddString.dta"
tab schyear _merge
keep if _merge == 1 | _merge == 3
drop _merge schaddid

gen distance_next_school = gis_eucDist
drop gis_eucDist calc_eucDist

/* Add distances to each exam school */

foreach sch in obr bla bls {
	if ("`sch'" == "obr") {
		gen schoolid = 1030
	}
	else if ("`sch'" == "bla") {
		gen schoolid = 1020
	}
	else if ("`sch'" == "bls") {
		gen schoolid = 1010
	}
	gsort schyear schoolid
	merge m:1 schyear schoolid using "$dir/Data/schadd2.dta"
	tab schyear _merge
	keep if _merge == 1 | _merge == 3
	drop _merge schoolid

	tostring addid schaddid, replace force
	merge m:1 addid schaddid using "$dir/Data/eucDistAddString.dta"
	tab schyear _merge
	keep if _merge == 1 | _merge == 3
	drop _merge schaddid
	
	gen distance_`sch' = gis_eucDist
	drop gis_eucDist calc_eucDist
}

/* Add peer means in each potential school(compute leave-one-out mean) */

foreach test in math4 eng4 math8 eng7 {
	foreach sch in obr bla bls {
		bysort schyear app_grade: egen temp = mean(peer_`test') if to_`sch' == 1
		bysort schyear app_grade: egen peer_`test'_`sch' = mean(temp)
		drop temp
		bysort schyear app_grade: egen temp = mean(peer_`test'_N) if to_`sch' == 1
		bysort schyear app_grade: egen peer_`test'_`sch'_N = mean(temp)
		drop temp
		replace peer_`test'_`sch' = (1/(peer_`test'_`sch'_N - 1))*(peer_`test'_`sch'_N*peer_`test'_`sch' - `test') if `test' != . & to_`sch' == 1
	}
}

/* Add peer means in the school a kid ends up in (compute leave-one-out mean) */
	
foreach x in eng4 math4 eng7 math7 eng8 math8 eng10 math10 {
	replace peer_`x' = (1/(peer_`x'_N - 1))*(peer_`x'_N*peer_`x' - `x') if `x' != .
}  

drop peer_*_N

gen peer_eng = peer_eng4 if app_grade == 7
replace peer_eng = peer_eng7 if app_grade == 9

gen peer_math = peer_math4 if app_grade == 7
replace peer_math = peer_math8 if app_grade == 9

foreach x in eng7 math7 eng8 math8 eng10 math10 {
	gen peer2_`x' = peer_`x'
}  

/* Add peers and compute peer quantiles in the school a kid ends up in */

gen temp_id = _n

save "$dir/Data/temporary_data.dta", replace
 
keep if app_grade != . & missing(next_school) == 0
keep temp_id studentno schyear app_grade next_school

gsort schyear app_grade next_school
joinby schyear app_grade next_school using "$dir/Data/peers_data.dta", unmatched(master)
tab schyear _merge
drop _merge

foreach x in math4 eng4 math8 eng7 {
	replace peers_`x' = . if studentno == peers_studentno
}

foreach x in math4 eng4 math8 eng7 {
	forval q = 25(50)75 {
		bysort temp_id schyear app_grade next_school: egen peer_q`q'_`x' = pctile(peers_`x'), p(`q')
	}
}

duplicates drop temp_id, force
keep temp_id peer_q*
compress
gsort temp_id
save "$dir/Data/peer_quantile_data.dta", replace
 
use "$dir/Data/temporary_data.dta", clear

gsort temp_id
merge temp_id using "$dir/Data/peer_quantile_data.dta"
tab schyear _merge
drop _merge

forval q = 25(50)75 {
	gen peer_q`q'_eng = peer_q`q'_eng4 if app_grade == 7
	replace peer_q`q'_eng = peer_q`q'_eng7 if app_grade == 9

	gen peer_q`q'_math = peer_q`q'_math4 if app_grade == 7
	replace peer_q`q'_math = peer_q`q'_math8 if app_grade == 9
}
drop peer_q*_eng4 peer_q*_eng7 peer_q*_math4 peer_q*_math8

/* Generate baseline test score variables */

gen base_eng = .
replace base_eng = eng4 if grade == 6
replace base_eng = eng7 if grade == 8

gen base_math = .
replace base_math = math4 if grade == 6
replace base_math = math8 if grade == 8

tab schyear grade
tab schyear grade if base_eng != .
tab schyear grade if base_math != .

/* Generate variables for gap between peer mean and own score */

gen peergap_eng = base_eng - peer_eng
gen peergap_math = base_math - peer_math

foreach sch in obr bla bls {
	gen peergap_eng_`sch' = .
	replace peergap_eng_`sch' = base_eng - peer_eng4_`sch' if grade == 6
	replace peergap_eng_`sch' = base_eng - peer_eng7_`sch' if grade == 8

	gen peergap_math_`sch' = .
	replace peergap_math_`sch' = base_math - peer_math4_`sch' if grade == 6
	replace peergap_math_`sch' = base_math - peer_math8_`sch' if grade == 8
}

/* Generate relative rank variables */

gen pcrank_eng = .
gen pcrank_math = .
quietly: levelsof schyear, local(years)
foreach yr in `years' {
	replace pcrank_eng = pcrank`yr'_eng4 if app_grade == 7 & schyear == `yr'
	replace pcrank_eng = pcrank`yr'_eng7 if app_grade == 9 & schyear == `yr'
	replace pcrank_math = pcrank`yr'_math4 if app_grade == 7 & schyear == `yr'
	replace pcrank_math = pcrank`yr'_math8 if app_grade == 9 & schyear == `yr'
}
drop pcrank19* pcrank20*

/* Generate relative ranks among applicants */

foreach test in eng math { 
	foreach sch in obr bla bls {
		bysort schyear app_grade: egen temp_n = count(`test'4) if app_`sch' == 1
		bysort schyear app_grade: egen temp_i = rank(`test'4) if app_`sch' == 1
		gen apprank_`test'_`sch' = (temp_i - 1)/(temp_n - 1) if app_`sch' == 1
		drop temp_*
	}
 }
 
/* Generate a variable for HS science using "Parag's rule" */

egen temp = rownonmiss(physhs biohs chemhs techhs)
tab temp
drop temp

gen sciehs = .
gen sciehs_level = .
gen yr_sciehs = .

replace yr_sciehs = yr_physhs if sciehs == .
replace sciehs_level = physhs_level if sciehs == .
replace sciehs = physhs if sciehs == .

replace yr_sciehs = yr_biohs if sciehs == .
replace sciehs_level = biohs_level if sciehs == .
replace sciehs = biohs if sciehs == .

replace yr_sciehs = yr_chemhs if sciehs == .
replace sciehs_level = chemhs_level if sciehs == .
replace sciehs = chemhs if sciehs == .

replace yr_sciehs = yr_techhs if sciehs == .
replace sciehs = techhs_level if sciehs == .
replace sciehs = techhs if sciehs == .

/* Generate variables for years spent in exam schools until a given test */

foreach test in math7 math8 math10 eng7 eng8 eng10 {
	gen exam_yrs_`test' = .
	gen obr_yrs_`test' = .
	gen bla_yrs_`test' = .
	gen bls_yrs_`test' = .
	forval i = 1997/2008 {
		local j = `i' + 1
		replace exam_yrs_`test' = yrs_in_exam_`j' if yr_`test' == `i'
		replace obr_yrs_`test' = yrs_in_obr_`j' if yr_`test' == `i'
		replace bla_yrs_`test' = yrs_in_bla_`j' if yr_`test' == `i'
		replace bls_yrs_`test' = yrs_in_bls_`j' if yr_`test' == `i'
	}
}

su exam_yrs_*
su obr_yrs_*
su bla_yrs_*
su bls_yrs_*

/* Generate variables for school attended when taking a given test */

foreach test in math7 math8 math10 eng7 eng8 eng10 scie8 sciehs satv satm satw sat psatr psatm psatw psat {
	gen sch_`test' = .
	forval i = 1997/2009 {
		local j = `i' + 1
		replace sch_`test' = school_`j' if yr_`test' == `i'
	}
}

/* Generate variables for school and year when in 12th grade for AP, and college outcomes */

forval j = 9/12 {
	gen naps_`j' = .
	forval i = 2005/2010 {
		replace naps_`j' = naps_`i' if grade_`i' == `j'
	}
}
su naps_*

gen sch_ap = .
gen yr_ap = .
forval i = 1996/2010 {
	replace sch_ap = school_`i' if grade_`i' == 12 & sch_ap == .
	replace yr_ap = `i' if grade_`i' == 12 & yr_ap == .
}

foreach x of varlist apnbr* apscr* take_apstem attend_* persist_* graduate_* barrons_* {
	gen sch_`x' = sch_ap
	gen yr_`x' = yr_ap
}
drop sch_ap yr_ap

/* Generate variables for having taken SAT and PSAT */

gen take_psat = (psat != .)
gen take_sat = (sat != .)

/* Generate the year and school when taking PSAT/SAT to be 10/12 */

foreach gr in 10 12 {
	gen sch_`gr' = .
	gen yr_`gr' = .
	forval i = 1996/2010 {
		replace sch_`gr' = school_`i' if grade_`i' == `gr' & sch_`gr' == .
		replace yr_`gr' = `i' if grade_`i' == `gr' & yr_`gr' == .
	}
}

gen sch_take_psat = sch_10
gen yr_take_psat = yr_10

gen sch_take_sat = sch_12
gen yr_take_sat = yr_12

drop sch_10 sch_12 yr_10 yr_12

/* Generate attrition indicators */

foreach test of varlist math7 math8 math10 eng7 eng8 eng10 scie8 sciehs satm satv satw sat take_sat psatr psatm psatw psat take_psat attend_* persist_* graduate_* barrons_* {
	gen have_`test' = 1 - missing(`test')
}

/* Generate indicators for having taken AP classes and set missing values to zero */

foreach x of varlist apnbr* apscr* {
	gen have_`x' = (`x' > 0 & `x' != .)
} 

foreach x of varlist apnbr* apscr* take_apstem {
	recode `x' . = 0
}

/* Generate variable for years in BPS after application */

forval j = 1/6 {
	gen inbps`j' = .
	local m = 2010 - `j'
	forval i = 1996/`m'{
		local k = `i' + `j'
		replace inbps`j' = (missing(school_`k') == 0) if schyear == `i'
	}
}

/* Generate variable for being in BPS X years after application */

gen bpsyrs = 0
forval i = 1996/2009 {
	local j = `i' + 1
	forval k = `j'/2010 {
		replace bpsyrs = bpsyrs + 1*(missing(school_`k') == 0) if schyear == `i'
	}
}

/* Generate variable for grade repetition */

gen graderep = .
forval i = 1996/2008 {
	forval j = 1998/2010 {
		local k = `j' - 1
		gen temp_`j' = (grade_`j' - grade_`k' == 0) if `k' > `i'
	}
	egen temp = rowmean(temp_*)
	replace graderep = (temp != 0) if temp != . & schyear == `i'
	drop temp*
}
replace graderep = . if (grade == 6 & schyear > 2003) | (grade == 8 & schyear > 2005)

tab schyear grade if app_exam != 1, su(graderep) nofr nost

/* Generate risk set indicator */

egen risk_set = group(app_grade schyear) if app_grade != .

/* Generate risk set and application preference interactions */

egen risk_set_prefs = group(app_grade schyear obr_pref bla_pref bls_pref), missing

/* Generate runvar controls */

foreach sch in obr bla bls {
	gen above_`sch' = (runvar_`sch' >= 0 & runvar_`sch' != .)
	gen runvar2_`sch' = runvar_`sch'^2
	gen runvar3_`sch' = runvar_`sch'^3
	gen intrunvar_`sch' = above_`sch'*runvar_`sch'
	gen intrunvar2_`sch' = above_`sch'*runvar2_`sch'
	gen intrunvar3_`sch' = above_`sch'*runvar3_`sch'
}

/* Generate indicators for the discontinuity samples */

foreach sch in obr bla bls {
	gen disc_`sch' = (runvar_`sch' >= -20 & runvar_`sch' <= 20)
}

/* Generate indicators for belonging to the school specific experiments */

foreach sch in obr bla bls {
	gen experiment_`sch' = (`sch'_pref == 1 | (`sch'_pref == 2 & above_choice1 == 0) | (`sch'_pref == 3 & above_choice1 == 0 & above_choice2 == 0))
}

/* Generate dummies for samples used for each outcome */

gen sample_7_math7 = (app_grade == 7 & schyear >= 2005 & schyear <= 2008)
gen sample_7_math8 = (app_grade == 7 & schyear >= 1999 & schyear <= 2007)
gen sample_7_math10 = (app_grade == 7 & schyear >= 1999 & schyear <= 2005)
gen sample_7_eng7 = (app_grade == 7 & schyear >= 2000 & schyear <= 2008)
gen sample_7_eng8 = (app_grade == 7 & schyear >= 2004 & schyear <= 2007)
gen sample_7_eng10 = (app_grade == 7 & schyear >= 1999 & schyear <= 2005)
gen sample_7_scie8 = (app_grade == 7 & schyear >= 2004 & schyear <= 2007)
gen sample_7_sciehs = (app_grade == 7 & schyear >= 2003 & schyear <= 2006)

foreach x in satv satm sat take_sat {
	gen sample_7_`x' = (app_grade == 7 & schyear >= 1999 & schyear <= 2004) 
}
gen sample_7_satw = (app_grade == 7 & schyear >= 2000 & schyear <= 2004)

foreach x in psatr psatm psatw psat take_psat {
	gen sample_7_`x' = (app_grade == 7 & schyear >= 2000 & schyear <= 2005) 
}

foreach x of varlist have_apscr* apnbr* apscr* take_apstem {
	gen sample_7_`x' = (app_grade == 7 & schyear >= 1999 & schyear <= 2004)
}

foreach x of varlist attend_* barrons_* {
	gen sample_7_`x' = (app_grade == 7 & schyear >= 1999 & schyear <= 2004)
}
foreach x of varlist persist_* {
	gen sample_7_`x' = (app_grade == 7 & schyear >= 1999 & schyear <= 2002)
}
gen sample_7_graduate_2yr = (app_grade == 7 & schyear >= 1999 & schyear <= 2002)
gen sample_7_graduate_4yr = (app_grade == 7 & schyear >= 1999 & schyear <= 2000)
gen sample_7_graduate_any = (app_grade == 7 & schyear >= 1999 & schyear <= 2000)

compress

gen sample_9_math10 = (app_grade == 9 & schyear >= 2001 & schyear <= 2007)
gen sample_9_eng10 = (app_grade == 9 & schyear >= 2001 & schyear <= 2007)
gen sample_9_sciehs = (app_grade == 9 & schyear >= 2005 & schyear <= 2007)

foreach x in satv satm sat take_sat {
	gen sample_9_`x' = (app_grade == 9 & schyear >= 2001 & schyear <= 2006) 
}
gen sample_9_satw = (app_grade == 9 & schyear >= 2002 & schyear <= 2006)

foreach x in psatr psatm psatw psat take_psat {
	gen sample_9_`x' = (app_grade == 9 & schyear >= 2002 & schyear <= 2007) 
}

foreach x of varlist have_apscr* apnbr* apscr* take_apstem {
	gen sample_9_`x' = (app_grade == 9 & schyear >= 2001 & schyear <= 2006)
}

foreach x of varlist attend_* barrons_* {
	gen sample_9_`x' = (app_grade == 9 & schyear >= 2001 & schyear <= 2006)
}
foreach x of varlist persist_* {
	gen sample_9_`x' = (app_grade == 9 & schyear >= 2001 & schyear <= 2004)
}
gen sample_9_graduate_2yr = (app_grade == 9 & schyear >= 2001 & schyear <= 2004)
gen sample_9_graduate_4yr = (app_grade == 9 & schyear >= 2001 & schyear <= 2002)
gen sample_9_graduate_any = (app_grade == 9 & schyear >= 2001 & schyear <= 2002)

compress

gen sample_7_math78 = (sample_7_math7 == 1 | sample_7_math8 == 1)
gen sample_7_eng78 = (sample_7_eng7 == 1 | sample_7_eng8 == 1)
compress

foreach x of varlist math10 eng10 sciehs satv satm satw sat take_sat psatr psatm psatw psat take_psat have_apscr* apnbr* apscr* take_apstem attend_* persist_* graduate_* barrons_* {
	gen sample_79_`x' = (sample_7_`x' == 1 | sample_9_`x' == 1)
}
gen sample_79_math7810 = (sample_7_math7 == 1 | sample_7_math8 == 1 | sample_7_math10 == 1 | sample_9_math10 == 1)
gen sample_79_eng7810 = (sample_7_eng7 == 1 | sample_7_eng8 == 1 | sample_7_eng10 == 1 | sample_9_eng10 == 1)
compress

/* Recode baseline, peer mean and relative rank as missing for "unusual" years */

tab schyear app_grade if base_eng != .
replace base_eng = . if (app_grade == 7 & schyear < 2000)

tab schyear app_grade if base_math != .
replace base_math = . if (app_grade == 7 & schyear < 2000) | (app_grade == 9 & schyear < 1998)
	 
tab schyear app_grade if peer_eng != .
tab schyear app_grade if peer_math != .
replace peer_eng = . if (app_grade == 7 & schyear < 2000) | (app_grade == 9 & schyear < 2002)
replace peer_math = . if (app_grade == 7 & schyear < 2000) | (app_grade == 9 & schyear < 1998)
forval q = 25(50)75 {
	replace peer_q`q'_eng = . if (app_grade == 7 & schyear < 2000) | (app_grade == 9 & schyear < 2002)
	replace peer_q`q'_math = . if (app_grade == 7 & schyear < 2000) | (app_grade == 9 & schyear < 1998)
}

tab schyear app_grade if pcrank_eng != .
	 
tab schyear app_grade if pcrank_math != .
replace peer_math = . if (app_grade == 9 & schyear < 1998)

/* Clean the data from some extra variables */

drop school_1996-grade_2012 yrs_in_exam_1996-yrs_in_bls_2012 ch1 ch2 ch3 inv_sch 
drop asn_sch accept awc awc0 awc1 awc2 above_choice* merge_check_id 
drop dob_id geo lepstatus firstlang biling sncode food dob in_bls in_bla in_obr in_exam merge_file age
drop peer_eng4-peer_math10 peer_math4_obr-peer_eng7_bls naps_* reg_num id1 id2

/* Save the analysis data*/

compress
save "$dir/Data/analysis_data.dta", replace

/* Close log file */

log close
