

*** This file constructs tables and plots for the paper using the analysis data.
*** Last update: 2013-05-09


/* General specifications */

clear all
set matsize 11000, permanently
set more off, permanently

*global dir = "h:/Research/Exam_Schools"
*adopath ++ "h:/ado/plus"

global dir = "//bbkinghome/rokkanen/Research/Exam_Schools"
adopath ++ "//bbkinghome/rokkanen/ado/plus"

ssc install ivreg2, replace
ssc install ranktest, replace


/********************/
/* DATA PREPARATION */
/********************/


/* Load data */

use "$dir/Data/analysis_data.dta", clear
desc

/* Drop redundant variables */

drop *gencomp* *aacutoff*


/**************************************************/
/* SAMPLE RESTRICTIONS FOR DESCRIPTIVE STATISTICS */
/**************************************************/


tab private, miss
keep if private != 1

tab yrs_in_exam_cur, miss
keep if yrs_in_exam_cur == 0 | yrs_in_exam_cur == .

tab schyear, miss
keep if (grade == 6 & schyear >= 1999 & schyear <= 2008) | (grade == 8 & schyear >= 2001 & schyear <= 2007)


/*************************************/
/* DESCRIPTIVE STATISTICS (TABLE A1) */
/*************************************/


tab schyear free, miss
tab schyear lep, miss
tab schyear special, miss

local variables = "female black hisp free lep special base_math base_eng"

foreach gr in 6 8 {

	preserve
	keep if grade == `gr'
	
	* All Boston *

	quietly: tabstat `variables' if app_exam != 1, save columns(stat)
	mat b =  (r(StatTotal))
	mat results_`gr' = b'
	
	foreach sch in obr bla bls {
	
		* Applicants *
	
		quietly: tabstat `variables' if app_`sch' == 1, save columns(stat)
		mat b =  (r(StatTotal))
		mat results_`gr' = (results_`gr', b')
		
		* Enrollees *
	
		quietly: tabstat `variables' if to_`sch' == 1, save columns(stat)
		mat b =  (r(StatTotal))
		mat results_`gr' = (results_`gr', b')
		
	}

	mat rownames results_`gr' = `variables'
	mat colnames results_`gr' = "all_bps" "app_obr" "to_obr" "app_bla" "to_bla" "app_bls" "to_bls"
	restore
}

mat results = (results_6\ results_8)
mat list results
mat2txt, matrix(results) saving("$dir/Paper_Material/Tables/descriptives2.txt") replace


/************************************************************/
/* ADDITIONAL SAMPLE RESTRICTIONS FOR PLOTS AND ESTIMATIONS */
/************************************************************/


/* Restrictions */

tab app_exam, miss
keep if app_exam == 1

tab app_grade grade, miss
keep if app_grade == 7 | app_grade == 9


/**********************************/
/* PLOTS (FIGURES 1, 2, 3, 4, A1) */
/**********************************/


/* Open log file */

capture: log close
log using "$dir/Results_and_Notes/plots.log", replace

/* Frequency plot */

local bin_min = `"-100"'
local bin_width = `"1"'
local bin_max = `"100"'

local xlabel = "-20(10)20"

local xtitle = "Normalized Score Rank"
local ylabel = ""
		
foreach gr in 7 9 79 {

	if `gr' == 7 {
		local restrictions = "app_grade == 7"
		local title = "Running Variable Density - Grade: 7"
	}
	else if `gr' == 9 {
		local restrictions = "app_grade == 9"
		local title = "Running Variable Density - Grade: 9"
	}
	else if `gr' == 79 {
		local restrictions = "app_gr == 7 | app_gr == 9"
		local title = "Running Variable Density - Grade: 7 & 9"
	}

	preserve
	keep if `restrictions'
		
	foreach sch in obr bla bls {

		if "`sch'" == "obr" {
			local subtitle = "O'Bryant"
		}
		else if "`sch'" == "bla" {
			local subtitle = "Latin Academy"
		}
		else if "`sch'" == "bls" {
			local subtitle = "Latin School"
		}

		histogram runvar_`sch' if (runvar_`sch' >= `bin_min' & runvar_`sch' < `bin_max' & experiment_`sch' == 1), ///
			density xlab(`xlabel') ylab(`ylabel') xline(0, lpattern(shortdash)) ytitle("") xtitle(" ") subtitle(`subtitle') legend(off) saving(temp_`sch', replace)
	}
	
	graph combine temp_obr.gph temp_bla.gph temp_bls.gph, cols(3) imargin(tiny) iscale(0.5) subtitle("`title'") ycommon xcommon saving("$dir/Paper_Material/Plots/separate_density_gr`gr'_lee", replace)
	graph2tex, epsfile("$dir/Paper_Material/Plots/separate_density_gr`gr'_lee")
	
	restore
}

/* Check sample construction */

local title = ""

foreach gr in 7 {
	preserve

	if ("`gr'" == "7") {
		keep if app_grade == 7
	}
	else if ("`gr'" == "9") {
		keep if app_grade == 9
	}
	else if ("`gr'" == "79") {
		keep if app_grade == 7 | app_grade == 9
	}
	
	if ("`gr'" == "7" | "`gr'" == "79") {
		local bin_min = `"-20"'
		local bin_width = `"1"'
		local bin_max = `"20"'
		local xlabel = "-20(10)20"
	}
	else {
		local bin_min = `"-10"'
		local bin_width = `"1"'
		local bin_max = `"10"'
		local xlabel = "-10(5)10"
	}

	foreach sch in obr bla bls {

		if ("`sch'" == "obr") {
			local subtitle = "O'Bryant"
		}
		else if ("`sch'" == "bla") {
			local subtitle = "Latin Academy"
		}
		else if ("`sch'" == "bls") {
			local subtitle = "Latin School"
		}
		
		local outcome = "experiment_`sch'"

		egen temp_x = cut(runvar_`sch') if (runvar_`sch' >= `bin_min' & runvar_`sch' < `bin_max'), at(`bin_min'(`bin_width')`bin_max')
		replace temp_x = temp_x  + (`bin_width'/2)
		bysort temp_x: egen temp_y = mean(`outcome')
		bysort temp_x: gen temp_i = _n

		capture: rdob_mod2 `outcome' runvar_`sch' if (`outcome' != . & runvar_`sch' != .) & disc_`sch' == 1, c(0) dm(1)
		local h_opt = r(h_opt)
		if (`h_opt' == .) {
			local h_opt = 5
		}
		
		disp _newline "`sch'" "-" `h_opt' _newline
		
		gen lambda_`sch' = (1 - abs(runvar_`sch'/`h_opt'))*(abs(runvar_`sch'/`h_opt') <= 1)
		
		quietly: reg `outcome' above_`sch' runvar_`sch' intrunvar_`sch' [aw = lambda_`sch'] if disc_`sch' == 1, robust
		eststo gr`gr'_experiment_`sch'_ik
	
		graph twoway (scatter temp_y temp_x if temp_i == 1 & temp_x >= `bin_min' & temp_x < `bin_max') ///
			(lpoly `outcome' runvar_`sch' if runvar_`sch' >= `bin_min' & runvar_`sch' < 0, lcolor(black) degree(1) kernel(triangle) bwidth(`h_opt')) ///
			(lpoly `outcome' runvar_`sch' if runvar_`sch' >= 0 & runvar_`sch' < `bin_max', lcolor(black) degree(1) kernel(triangle) bwidth(`h_opt')), ///
			xlab(`xlabel') ylab(`ylabel') xline(0, lpattern(shortdash)) ytitle("") xtitle("") subtitle(`subtitle') legend(off) saving(temp_`sch', replace)
		drop temp_x temp_y temp_i

	}

	graph combine temp_obr.gph temp_bla.gph temp_bls.gph, cols(3) imargin(tiny) iscale(0.5) subtitle("`title'") ycommon xcommon saving("$dir/Paper_Material/Plots/separate_experiment_gr`gr'_lee", replace)
	graph2tex, epsfile("$dir/Paper_Material/Plots/separate_experiment_gr`gr'_lee")
	
	esttab gr`gr'_experiment_*_ik, b(3) se(3) keep(above_*) star(* 0.1 ** 0.05 *** 0.01)
	
	restore
}

/* Lee (2008) -style plots with IK fit */

foreach sample in all /* black hisp blackhisp */ {
	foreach gr in 7 9 79 {

		if (`gr' == 7) {
			local variables1 = "offer_yn to_exam yrs_in_exam_tot peer_math peer_eng pcrank_math pcrank_eng distance_next_school"
			local variables2 = "math7 math8 math78 math10 eng7 eng8 eng78 eng10 sciehs take_sat sat take_psat psat apnbrall apnbr500 apscrall apscr500 take_apstem"
			local variables3 = "attend_2yr attend_4yr attend_any persist_2yr persist_4yr persist_any graduate_2yr graduate_4yr graduate_any"
			local variables4 = "barrons_MC barrons_HC barrons_VC barrons_C barrons_LC barrons_S"
			local variables5 = "peer_q25_math peer_q75_math peer_q25_eng peer_q75_eng prop_blackhisp"
		}
		else if (`gr' == 9) {
			local variables1 = "offer_yn to_exam yrs_in_exam_tot peer_math peer_eng pcrank_math pcrank_eng distance_next_school"
			local variables2 = "math10 eng10 sciehs take_sat sat take_psat psat apnbrall apnbr500 apscrall apscr500 take_apstem"
			local variables3 = "attend_2yr attend_4yr attend_any persist_2yr persist_4yr persist_any graduate_2yr graduate_4yr graduate_any"
			local variables4 = "barrons_MC barrons_HC barrons_VC barrons_C barrons_LC barrons_S"
			local variables5 = "peer_q25_math peer_q75_math peer_q25_eng peer_q75_eng prop_blackhisp"
		}
		else {
			local variables1 = "offer_yn to_exam yrs_in_exam_tot peer_math peer_eng pcrank_math pcrank_eng distance_next_school"
			local variables2 = "math10 eng10 sciehs take_sat sat take_psat psat apnbrall apnbr500 apscrall apscr500 take_apstem"
			local variables3 = "attend_2yr attend_4yr attend_any persist_2yr persist_4yr persist_any graduate_2yr graduate_4yr graduate_any"
			local variables4 = "barrons_MC barrons_HC barrons_VC barrons_C barrons_LC barrons_S"
			local variables5 = "peer_q25_math peer_q75_math peer_q25_eng peer_q75_eng prop_blackhisp"
		}
		if (`gr' == 7 | `gr' == 79) {
			local bin_min = `"-20"'
			local bin_width = `"1"'
			local bin_max = `"20"'
			local xlabel = "-20(10)20"
		}
		else {
			local bin_min = `"-10"'
			local bin_width = `"1"'
			local bin_max = `"10"'
			local xlabel = "-10(5)10"
		}
		
		mat table_`gr' = J(1, 3, .)
		
		foreach outcome in `variables1' `variables2' `variables3' `variables4' `variables5'  {
		
			if ("`outcome'" == "offer_yn") {
				local ylabel = "0(0.2)1"
				local ytitle = "Offer Rate"
			}
			if ("`outcome'" == "to_exam") {
				local ylabel = "0(0.2)1"
				local ytitle = "Enrollment Rate"
			}
			else if ("`outcome'" == "yrs_in_exam_tot") {
				local ylabel = "0(1)5"
				local ytitle = "Years Enrolled"
			}		
			else if ("`outcome'" == "peer_math" | "`outcome'" == "peer_eng") {
				local ylabel = "-0.5(0.5)2.5"
				local ytitle = "Peer Mean"
			}
			else if ("`outcome'" == "pcrank_eng" | "`outcome'" == "pcrank_math") {
				local ylabel = "0(0.2)1"
				local ytitle = "Peer Rank"
			}
			else if ("`outcome'" == "prop_blackhisp") {
				local ylabel = "0(0.2)1"
				local ytitle = "Peer Proportion Back/Hispanic"
			}
			else if ("`outcome'" == "math7" | "`outcome'" == "math8" | "`outcome'" == "math78" | "`outcome'" == "math10" | "`outcome'" == "eng7" | "`outcome'" == "eng8" | "`outcome'" == "eng78" | "`outcome'" == "eng10" | "`outcome'" == "biohs" | "`outcome'" == "sciehs" | "`outcome'" == "sat" | "`outcome'" == "satv" | "`outcome'" == "satm" | "`outcome'" == "satw") {
				local ylabel = "-0.5(0.5)2.5"
				local ytitle = "Standardized Score"
			}
			else if ("`outcome'" == "sat" | "`outcome'" == "satv" | "`outcome'" == "satm" | "`outcome'" == "satw" | "`outcome'" == "psat" | "`outcome'" == "psatr" | "`outcome'" == "psatm" | "`outcome'" == "psatw") {
				local ylabel = "-0.5(0.5)2.5"
				local ytitle = "Standardized Score"
			}
			else if ("`outcome'" == "take_sat" | "`outcome'" == "take_psat") {
				local ylabel = "0(0.2)1"
				local ytitle = "Proportion Taking Exam"
			}
			else if ("`outcome'" == "apnbrall" | "`outcome'" == "apnbrall2") {
				local ylabel = "0(1)4"
				local ytitle = "Number of Courses"
			}
			else if ("`outcome'" == "apnbr500") {
				local ylabel = "0(0.5)3"
				local ytitle = "Number of Courses"
			}
			else if ("`outcome'" == "apnbr1000") {
				local ylabel = "0(0.5)2"
				local ytitle = "Number of Courses"
			}
			else if ("`outcome'" == "apscrall" | "`outcome'" == "apscrall2") {
				local ylabel = "0(2)16"
				local ytitle = "Sum of Scores"
			}
			else if ("`outcome'" == "apscr500") {
				local ylabel = "0(2)10"
				local ytitle = "Sum of Scores"
			}
			else if ("`outcome'" == "apscr1000") {
				local ylabel = "0(1)6"
				local ytitle = "Sum of Scores"
			}
			else if (substr("`outcome'", 1, 7) == "attend_" | substr("`outcome'", 1, 8) == "persist_" | substr("`outcome'", 1, 9) == "graduate_" | substr("`outcome'", 1, 8) == "barrons_") {
				local ylabel = "0(0.2)1"
			}
			else if ("`outcome'" == "bpsyrs") {
				local ylabel = "3(0.5)6"
			}
			else if ("`outcome'" == "inbps1" | "`outcome'" == "inbps2" | "`outcome'" == "inbps3" | "`outcome'" == "inbps4" | "`outcome'" == "inbps5" | "`outcome'" == "inbps6") {
				local ylabel = "0(0.2)1"
			}
			else if (substr("`outcome'", 1, 8) == "peer_q25") {
				local ylabel = "-1(0.5)1.5"
			}
			else if (substr("`outcome'", 1, 8) == "peer_q75") {
				local ylabel = "0(0.5)3"
			}
			else if (substr("`outcome'", 1, 8) == "distance") {
				local ylabel = "1(1)5"
				local ytitle = "Distance to School"
			}
			else {
				local ylabel = ""
				local ytitle = ""
			}
			
			local title = ""
			local xtitle = "Standardized Running Variable"
			
			disp _newline `gr' "-" "`outcome'" _newline

			preserve	
			if (`gr' != 79) {
				keep if app_grade == `gr'
			}
			else {
				keep if app_grade == 7 | app_grade == 9
			}
			capture: keep if sample_`gr'_`outcome' == 1
			disp _rc
			
			if ("`sample'" == "black") {
				keep if black == 1
			}
			else if ("`sample'" == "hisp") {
				keep if hisp == 1
			}
			else if ("`sample'" == "blackhisp") {
				keep if black == 1 | hisp == 1
			}
			
			if ("`outcome'" == "math78" | "`outcome'" == "eng78") {
				if ("`outcome'" == "math78") {
					rename math7 score_math7
					rename math8 score_math8
				}
				if ("`outcome'" == "eng78") {
					rename eng7 score_eng7
					rename eng8 score_eng8
				}
				keep score_* runvar_* disc_* experiment_*
				gen id = _n
				reshape long score_, i(id) j(test) string
				rename score_ `outcome'
				compress
			}
			
			mat kuv = (., ., ., .)
			foreach i in 10 20 {
				mat rivi = (.)
				foreach sch in obr bla bls {
					quietly: count if `outcome' != . & (runvar_`sch' >= -`i' & runvar_`sch' < `i') & experiment_`sch' == 1
					mat rivi = (rivi, r(N))
				}
				quietly: count if `outcome' != . & ((runvar_obr >= -`i' & runvar_obr < `i' & experiment_obr == 1) | (runvar_bla >= -`i' & runvar_bla < `i' & experiment_bla == 1) | (runvar_bls >= -`i' & runvar_bls < `i' & experiment_bls == 1))
				mat rivi = (rivi, r(N))
				mat kuv = (kuv\ rivi[1..., 2...])
			}
			mat kuv = kuv[2..., 1...]
			mat rownames kuv = "10" "20"
			mat colnames kuv = "OBR" "BLA" "BLS" "ANY"
			mat list kuv
			
			foreach sch in obr bla bls {
		
				if "`sch'" == "obr" {
					local subtitle = "O'Bryant"
				}
				else if "`sch'" == "bla" {
					local subtitle = "Latin Academy"
				}
				else if "`sch'" == "bls" {
					local subtitle = "Latin School"
				}

				egen temp_x = cut(runvar_`sch') if (runvar_`sch' >= `bin_min' & runvar_`sch' < `bin_max') & experiment_`sch' == 1, at(`bin_min'(`bin_width')`bin_max')
				replace temp_x = temp_x  + (`bin_width'/2)
				bysort temp_x: egen temp_y = mean(`outcome') if experiment_`sch' == 1
				bysort temp_x: gen temp_i = _n

				capture: rdob_mod2 `outcome' runvar_`sch' if (`outcome' != . & runvar_`sch' != .) & disc_`sch' == 1 & experiment_`sch' == 1, c(0) dm(1)
				local h_opt = r(h_opt)
				scalar bw_`gr'_`outcome'_`sch' = `h_opt'
				if (`h_opt' == .) {
					local h_opt = 10
				}
				
				disp _newline "`sch'" "-" `h_opt' _newline
				
				graph twoway (scatter temp_y temp_x if temp_i == 1 & temp_x >= `bin_min' & temp_x < `bin_max' & experiment_`sch' == 1) ///
					(lpoly `outcome' runvar_`sch' if runvar_`sch' >= `bin_min' & runvar_`sch' < 0 & experiment_`sch' == 1, lcolor(black) degree(1) kernel(triangle) bwidth(`h_opt')) ///
					(lpoly `outcome' runvar_`sch' if runvar_`sch' >= 0 & runvar_`sch' < `bin_max' & experiment_`sch' == 1, lcolor(black) degree(1) kernel(triangle) bwidth(`h_opt')), ///
					xlab(`xlabel') ylab(`ylabel') xline(0, lpattern(shortdash)) ytitle("") xtitle("") subtitle(`subtitle') legend(off) saving(temp_`sch', replace)
				drop temp_x temp_y temp_i
			
			}
			
			graph combine temp_obr.gph temp_bla.gph temp_bls.gph, cols(3) imargin(tiny) iscale(0.5) subtitle("`title'") l1title(`ytitle', size(small)) b1title(`xtitle', size(small)) ycommon xcommon saving("$dir/Paper_Material/Plots/separate_`sample'_`outcome'_gr`gr'_lee", replace)
			graph2tex, epsfile("$dir/Paper_Material/Plots/separate_`sample'_`outcome'_gr`gr'_lee")

			restore
		}
		
		if (`gr' == 7) {
			local restrictions = "app_grade == 7"
		}
		else if (`gr' == 9) {
			local restrictions = "app_grade == 9"
		}
		else if (`gr' == 79) {
			local restrictions = "app_grade == 7 | app_grade == 9"
		}
		
		preserve
		keep if `restrictions'
		
		gen to_yn = to_exam
		gen yrs_in_tot_obr = yrs_in_obr_tot
		gen yrs_in_tot_bla = yrs_in_bla_tot
		gen yrs_in_tot_bls = yrs_in_bls_tot
		
		foreach outcome in offer to yrs_in_tot {
			
			if ("`outcome'" == "yrs_in_tot") {
				local ylabel = "0(1)5"
				local ytitle = "Years Enrolled"
			}
			else if ("`outcome'" == "offer") {
				local ylabel = "0(0.2)1"
				local ytitle = "Offer Rate"
			}
			else if ("`outcome'" == "to") {
				local ylabel = "0(0.2)1"
				local ytitle = "Enrollment Rate"
			}
			
			local title = ""
			local xtitle = "Standardized Running Variable"

			foreach sch in obr bla bls {
			
				if "`sch'" == "obr" {
					local subtitle = "O'Bryant"
				}
				else if "`sch'" == "bla" {
					local subtitle = "Latin Academy"
				}
				else if "`sch'" == "bls" {
					local subtitle = "Latin School"
				}
				
				gen outcome = `outcome'_`sch'
				
				egen temp_x = cut(runvar_`sch') if (runvar_`sch' >= `bin_min' & runvar_`sch' < `bin_max') & experiment_`sch' == 1, at(`bin_min'(`bin_width')`bin_max')
				replace temp_x = temp_x  + (`bin_width'/2)		
				bysort temp_x: egen temp_y = mean(outcome) if experiment_`sch' == 1
				bysort temp_x: gen temp_i = _n

				capture: rdob_mod2 outcome runvar_`sch' if (outcome != . & runvar_`sch' != .) & disc_`sch' == 1 & experiment_`sch' == 1, c(0) dm(1)
				local h_opt = r(h_opt)
				scalar bw_`gr'_`outcome'_`sch' = `h_opt'
				if (`h_opt' == .) {
					local h_opt = 10
				}
				disp _newline "`sch'" "-" `h_opt' _newline
				
				graph twoway (scatter temp_y temp_x if temp_i == 1 & temp_x >= `bin_min' & temp_x < `bin_max' & experiment_`sch' == 1, legend(off)) ///
					(lpoly outcome runvar_`sch' if runvar_`sch' >= `bin_min' & runvar_`sch' < 0 & experiment_`sch' == 1, lcolor(black) degree(1) kernel(triangle) bwidth(`h_opt') legend(off)) ///
					(lpoly outcome runvar_`sch' if runvar_`sch' >= 0 & runvar_`sch' < `bin_max' & experiment_`sch' == 1, lcolor(black) degree(1) kernel(triangle) bwidth(`h_opt') legend(off)), ///
					xlab(`xlabel') ylab(`ylabel') xline(0, lpattern(shortdash)) ytitle("") xtitle("") subtitle(`subtitle') legend(off) saving(temp_`sch', replace)	
				drop temp_x temp_y temp_i outcome
			
			}
			
			graph combine temp_obr.gph temp_bla.gph temp_bls.gph, cols(3) imargin(tiny) iscale(0.5) subtitle("`title'") l1title(`ytitle', size(small)) b1title(`xtitle', size(small)) ycommon xcommon saving("$dir/Paper_Material/Plots/separate_`sample'_`outcome'_specific_gr`gr'_lee", replace)
			graph2tex, epsfile("$dir/Paper_Material/Plots/separate_`sample'_`outcome'_specific_gr`gr'_lee")
		
		}	
		restore
		
		* Bandwidth table *
		
		foreach x in `variables1' `variables2' `variables3' `variables4' `variables5' offer to yrs_in_tot {
			mat row = (.)
			foreach sch in obr bla bls {
				capture: mat row = (row, bw_`gr'_`x'_`sch')				
			}
			if _rc == 0 {
				mat row = row[1...,2...]
				mat rownames row = "`gr'_`x'"
				mat table_`gr' = (table_`gr' \ row)
			}
		}
		mat table_`gr' = table_`gr'[2..., 1...]
		mat colnames table_`gr' = obr bla bls
	}
}

mat list table_7
mat list table_9
mat list table_79

/* Close log file */

log close


/***********************************************************************/
/* ESTIMATIONS & ATTRITION ANALYSIS (TABLES 3, 4, 5, 6, 9, A2, A4, A5) */
/***********************************************************************/


/* Save unstacked version of the data */

save "$dir/Data/unstacked_data.dta", replace

/* Drop redundant variables */

drop curr_sch race sex private app_bls app_bla app_obr isee_score
drop school grade yrs_in_bls_tot yrs_in_bla_tot yrs_in_obr_tot yrs_in_exam_tot yrs_in_bls_cur yrs_in_bla_cur yrs_in_obr_cur yrs_in_exam_cur
drop *raw *apnbrgrade4 *apnbrgrade5 next_school next_grade exam_school next_school_name peer_q*
drop *apnbrall2 *apscrall2 *apnbr1000 *apscr1000
drop age_* p_licensed n_core p_core* total_fte st_ratio inexam* inbps* innext* in9sch* offer_* graderep
drop bls_comp bla_comp obr_comp bls_rank bla_rank obr_rank *_cutoff *_stdcomp 
drop gpa_score verb_score quan_score read_score math_score female_a black_a white_a asian_a hisp_a other_a
drop eng4 *_eng4 eng4_* math4 *_math4 math4_* chemhs physhs techhs biohs *_chemhs *_physhs *_techhs *_biohs chemhs_* physhs_* techhs_* biohs_*
drop satv satm satw *_satv *_satm *_satw psatr psatm psatw *_psatr *_psatm *_psatw 
drop *barrons_MC *barrons_VC *barrons_LC *barrons_S *attend_2yr *persist_2yr *graduate_2yr
drop isee scie8 scie8_level obr_yrs_* bla_yrs_* bls_yrs_*

/* Prepare data for reshaping */

foreach test of varlist math7 math8 math10 eng7 eng8 eng10 sciehs sat take_sat psat take_psat apnbr* apscr* take_apstem attend_* persist_* graduate_* barrons_* {
	rename `test' score_`test'
	capture: rename `test'_level level_`test'
}

foreach test in math7 math8 math10 {
	gen base_`test' = base_math
}
foreach test in eng7 eng8 eng10 {
	gen base_`test' = base_eng
}
foreach test in biohs sciehs sat psat take_sat take_psat apnbrall apscrall apnbr500 apscr500 take_apstem attend_any persist_any graduate_any attend_4yr persist_4yr graduate_4yr barrons_C barrons_HC {
	egen base_`test' = rowmean(base_math base_eng)
}
drop base_math base_eng

foreach test in math7 math8 math10 {
	gen peer_`test' = peer_math
}
foreach test in eng7 eng8 eng10 {
	gen peer_`test' = peer_eng
}
foreach test in biohs sciehs sat psat take_sat take_psat apnbrall apscrall apnbr500 apscr500 take_apstem attend_any persist_any graduate_any attend_4yr persist_4yr graduate_4yr barrons_C barrons_HC {
	egen peer_`test' = rowmean(peer_math peer_eng)
}
drop peer_math peer_eng

foreach test in math7810 {
	gen peergap_`test' = peergap_math
	foreach sch in obr bla bls {
		gen peergap_`test'_`sch' = peergap_math_`sch'
		gen apprank_`test'_`sch' = apprank_math_`sch'
	}
}
foreach test in eng7810 {
	gen peergap_`test' = peergap_eng
	foreach sch in obr bla bls {
		gen peergap_`test'_`sch' = peergap_eng_`sch'
		gen apprank_`test'_`sch' = apprank_eng_`sch'
	}
}
drop peergap_math peergap_math_* peergap_eng peergap_eng_* apprank_math_* apprank_eng_*

/* Add NYC data */

rename schyear schyear_bos
rename to_exam to_exam_bos
append using "$dir/Data/NYC_2SLS_data.dta"
rename schyear schyear_nyc
rename to_exam to_exam_nyc
destring schyear_* to_exam_*, replace

rename peer_mean_MathBG peer_MathBG
rename peer_mean_EXRL peer_EXRL

gen schyear = schyear_bos
replace schyear = schyear_nyc if schyear == .

gen to_exam = to_exam_bos
replace to_exam = to_exam_nyc if to_exam == .

replace prop_blackhisp = peer_blkhisp if prop_blackhisp == .

foreach sch in K89S M89S X89S {
	gen experiment_`sch' = 1
}

drop schyear_* to_exam_* *_HXR *_HXRA *_MXRA *_MXRB *_MathAE *_SXRK peer_blkhisp peer_frpl peer_fem

/* Reshape data */

gen id = _n
destring yr_*, replace
tostring sch_*, replace

compress
reshape long score_ level_ yr_ sch_ peer_ base_ exam_yrs_ have_, i(id) j(test) string
replace sch_ = "" if sch_ == "."
compress

/* Generate sample indicators for NYC outcomes + modify them for BOS */

foreach test in math7 math8 math10 eng7 eng8 eng10 sciehs sat take_sat psat take_psat apnbrall apnbr500 apscrall apscr500 take_apstem attend_any persist_any graduate_any attend_4yr persist_4yr graduate_4yr barrons_C barrons_HC {
	replace sample_7_`test' = (sample_7_`test' == 1 & test == "`test'")
}
replace sample_7_math78 = (sample_7_math7  == 1 | sample_7_math8)
replace sample_7_eng78 = (sample_7_eng7  == 1 | sample_7_eng8)
compress

foreach test in math10 eng10 sciehs sat take_sat psat take_psat apnbrall apnbr500 apscrall apscr500 take_apstem attend_any persist_any graduate_any attend_4yr persist_4yr graduate_4yr barrons_C barrons_HC {
	replace sample_9_`test' = (sample_9_`test' == 1 & test == "`test'")
}
compress

foreach test in math10 eng10 sciehs sat take_sat psat take_psat apnbrall apnbr500 apscrall apscr500 take_apstem attend_any persist_any graduate_any attend_4yr persist_4yr graduate_4yr barrons_C barrons_HC {
	replace sample_79_`test' = (sample_7_`test' == 1 | sample_9_`test' == 1)
}
replace sample_79_math7810 = (sample_7_math7 == 1 | sample_7_math8 == 1 | sample_7_math10 == 1 | sample_9_math10 == 1)
replace sample_79_eng7810 = (sample_7_eng7 == 1 | sample_7_eng8 == 1 | sample_7_eng10 == 1 | sample_9_eng10 == 1)
compress

foreach test in EXRL MathBG {
	gen sample_9_`test' = (sample_`test' == 1 & test == "`test'")
}
drop sample_EXRL sample_MathBG
compress

gen sample_9_math10N = (sample_9_math10 == 1 | sample_9_MathBG == 1)
gen sample_9_eng10N = (sample_9_eng10 == 1 | sample_9_EXRL == 1)
gen sample_79_math10N = (sample_79_math10 == 1 | sample_9_MathBG == 1)
gen sample_79_eng10N = (sample_79_eng10 == 1 | sample_9_EXRL == 1)
gen sample_79_math7810N = (sample_79_math7810 == 1 | sample_9_MathBG == 1)
gen sample_79_eng7810N = (sample_79_eng7810 == 1 | sample_9_EXRL == 1)
compress

/* Generate variable for city */

gen city = "BOS" if app_exam == 1
replace city = "NYC" if app_exam != 1

/* Assign application grade for NYC kids */

replace app_grade = 9 if city == "NYC"
compress

/* Generate risk set variable that takes into account city */

egen risk_set_prefs2 = group(city risk_set_prefs), missing
drop risk_set_prefs
rename risk_set_prefs2 risk_set_prefs

/* Open log file */

capture: log close
log using "$dir/Results_and_Notes/estimations and attrition analysis.log", replace

/* Estimations */

local samples = "all"
*local samples = "all black hisp blackhisp nonblackhisp upperhalf upperquart"
*local samples = "male female"
*local samples = "blackhisp nonblackhisp"
*local samples = "blackhisp upperhalf upperquart"
*local samples = "lowerhalf"

foreach sample of local samples {

	disp _newline "Sample: `sample'"
	
	local demographics = "black asian hisp other female free"
	local isee_scores = "verb quan read math"

	foreach gr in 7 9 79 {
		if (`gr' == 7 & "`sample'" == "all") {
			local variables1 = "math7 math8 math10 eng7 eng8 eng10 math78 eng78"
			local variables2 = "take_psat psat take_sat sat apnbrall apnbr500 apscrall apscr500 sciehs take_apstem"
			local variables3 = "attend_any persist_any graduate_any attend_4yr persist_4yr graduate_4yr barrons_C barrons_HC"
		}
		else if (`gr' == 7 & "`sample'" != "all") {
			local variables1 = "math7 math8 math10 eng7 eng8 eng10 math78 eng78"
			local variables2 = "take_psat psat take_sat sat apnbrall apnbr500 apscrall apscr500 sciehs take_apstem"
			local variables3 = "attend_any persist_any graduate_any attend_4yr persist_4yr graduate_4yr barrons_C barrons_HC"
		}
		else if (`gr' == 9 & "`sample'" == "all") {
			local variables1 = "math10 eng10 MathBG EXRL"
			local variables2 = "take_psat psat take_sat sat apnbrall apnbr500 apscrall apscr500 sciehs take_apstem"
			local variables3 = "attend_any persist_any graduate_any attend_4yr persist_4yr graduate_4yr barrons_C barrons_HC"
		}
		else if (`gr' == 9 & "`sample'" != "all") {
			local variables1 = "math10 eng10"
			local variables2 = "take_psat psat take_sat sat apnbrall apnbr500 apscrall apscr500 sciehs take_apstem"
			local variables3 = "attend_any persist_any graduate_any attend_4yr persist_4yr graduate_4yr barrons_C barrons_HC"
		}
		else if (`gr' == 79 & "`sample'" == "all") {
			local variables1 = "math10 eng10 math7810 eng7810 math7810N eng7810N"
			local variables2 = "take_psat psat take_sat sat apnbrall apnbr500 apscrall apscr500 sciehs take_apstem"
			local variables3 = "attend_any persist_any graduate_any attend_4yr persist_4yr graduate_4yr barrons_C barrons_HC"
		}
		else if (`gr' == 79 & "`sample'" != "all") {
			local variables1 = "math10 eng10 math7810 eng7810"
			local variables2 = "take_psat psat take_sat sat apnbrall apnbr500 apscrall apscr500 sciehs take_apstem"
			local variables3 = "attend_any persist_any graduate_any attend_4yr persist_4yr graduate_4yr barrons_C barrons_HC"
		}
			
		foreach outcome in `variables1' `variables2' `variables3' {
		
			est clear
			estwrite _all using "$dir/Paper_Material/Tables/Estimates/ols_`sample'_gr`gr'_`outcome'.sters", replace
			estwrite _all using "$dir/Paper_Material/Tables/Estimates/rf_`sample'_gr`gr'_`outcome'.sters", replace
			estwrite _all using "$dir/Paper_Material/Tables/Estimates/attrition_`sample'_gr`gr'_`outcome'.sters", replace
			estwrite _all using "$dir/Paper_Material/Tables/Estimates/2sls_`sample'_gr`gr'_`outcome'.sters", replace
		
			if ("`outcome'" == "MathBG" | "`outcome'" == "EXRL") {
				local nyc = 1
				local bos = 0
			}
			else if ("`outcome'" == "math10N" | "`outcome'" == "eng10N" | "`outcome'" == "math7810N" | "`outcome'" == "eng7810N") {
				local nyc = 1
				local bos = 1
			}
			else {
				local nyc = 0
				local bos = 1
			}

			if ("`outcome'" == "math78" | "`outcome'" == "eng78" | "`outcome'" == "math7810" | "`outcome'" == "eng7810" | "`outcome'" == "math7810N" | "`outcome'" == "eng7810N") {		
				local se = "cluster(clusterid id)"
				local se_stc = "cluster(clusterid id)"
			}
			else {
				local se = "cluster(clusterid)"
				local se_stc = "cluster(clusterid id)"
			}
		
			disp _newline "`sample'" "-" `gr' "-" "`outcome'" _newline

			preserve
			if ("`outcome'" == "attr_joint") {
				keep if (sample_79_math7810 == 1 | sample_79_eng7810 == 1)
			}
			else {
				keep if (sample_`gr'_`outcome' == 1)
			}
			drop sample_*
			
			* Full sample vs. subsample selection *
			
			if ("`sample'" == "male") {
				keep if (female == 0)
			}
			else if ("`sample'" == "female") {
				keep if (female == 1)
			}
			else if ("`sample'" == "black") {
				keep if (black == 1)
			}
			else if ("`sample'" == "hisp") {
				keep if (hisp == 1)
			}
			else if ("`sample'" == "blackhisp") {
				keep if (black == 1 | hisp == 1)
			}
			else if ("`sample'" == "nonblackhisp") {
				drop if (black == 1 | hisp == 1)
			}
			else if ("`sample'" == "upperhalf") {
				bysort app_grade schyear: egen temp = pctile(base_), p(50)
				keep if (base_ >= temp & base_ != .)
				drop temp
			}
			else if ("`sample'" == "upperquart") {
				bysort app_grade schyear: egen temp = pctile(base_), p(75)
				keep if (base_ >= temp & base_ != .)
				drop temp
			}
			else if ("`sample'" == "lowerhalf") {
				bysort app_grade schyear: egen temp = pctile(base_), p(50)
				keep if (base_ < temp & base_ != .)
				drop temp
			}
			else if ("`sample'" == "lowerquart") {
				bysort app_grade schyear: egen temp = pctile(base_), p(25)
				keep if (base_ < temp & base_ != .)
				drop temp
			}
			
			quietly: count
			disp _newline "Sample size: " r(N) _newline
			
			if (`nyc' == 1 & `bos' == 1) {
				local schools = "obr bla bls K89S X89S M89S" 
			}
			else if (`nyc' == 1 & `bos' == 0) {
				local schools = "K89S X89S M89S"
			}
			else {
				local schools = "obr bla bls"
			}
			
			egen clusterid = group(city sch_ yr_)

			egen pool = group(city app_grade test)
			quietly: su pool
			local min = r(min)
			local max = r(max)
			forval i = `min'/`max' {
				foreach x in above runvar runvar2 runvar3 intrunvar intrunvar2 intrunvar3 {
					foreach sch in `schools' {
						gen pool`i'_`x'_`sch' = (pool == `i')*`x'_`sch'
					}
				}
			}
			
			egen pool_risk_set_prefs = group(pool risk_set_prefs)
			quietly: tab pool_risk_set_prefs, gen(pool_risk_set_prefs_)
			
			egen pool_test_yr = group(pool yr_)
			quietly: tab pool_test_yr, gen(pool_test_yr_)
			
			* Mean of outcome and baseline score *
			
			if (`nyc' == 0) {			
				quietly: su score_ if (disc_obr == 1 | disc_bla == 1 | disc_bls == 1)
				global mean_`outcome'_gr`gr' = r(mean)
				global n_`outcome'_gr`gr' = r(N)
				
				quietly: su have_ if (disc_obr == 1 | disc_bla == 1 | disc_bls == 1)
				global mean_h_`outcome'_gr`gr' = r(mean)
				global n_h_`outcome'_gr`gr' = r(N)

				quietly: su base_ if (disc_obr == 1 | disc_bla == 1 | disc_bls == 1)
				global mean_base_`outcome'_gr`gr' = r(mean)
				global n_base_`outcome'_gr`gr' = r(N)

				gen temp = (level_ == 1) if level_ != .
				quietly: su temp if (disc_obr == 1 | disc_bla == 1 | disc_bls == 1)
				global mean_level_`outcome'_gr`gr' = r(mean) 
				global n_level_`outcome'_gr`gr' = r(N)
				drop temp
			}
			
			* School-specific models *

			foreach sch in `schools' {
			
				disp _newline "`sch'" _newline

				local fixed_effects = "pool_risk_set_prefs_* pool_test_yr_*"
				local fixed_effects_attrition = "pool_risk_set_prefs_*"
	
				local ik_runvar_controls = "pool*_*runvar_`sch'"
				local cbc_runvar_controls = "pool*_*runvar_`sch' pool*_*runvar2_`sch' pool*_*runvar3_`sch'"
				
				* IK estimation *
								
				capture: rdob_mod2 score_ runvar_`sch' if experiment_`sch' == 1 & disc_`sch' == 1, c(0) dm(1)
				local h_opt = r(h_opt)
				if (`h_opt' == .) {
					local h_opt = 10
				}
				scalar bw_`gr'_`sch'_`outcome' = `h_opt'

				gen lambda_`sch' = (1 - abs(runvar_`sch'/`h_opt'))*(abs(runvar_`sch'/`h_opt') <= 1)
				gen extrapol_lambda_`sch' = (1 - abs((runvar_`sch' - above_`sch')/`h_opt'))*(abs((runvar_`sch' - above_`sch')/`h_opt') <= 1)				
				
				if (`gr' == 79 | "`outcome'" == "math78" | "`outcome'" == "eng78" | "`outcome'" == "math10N" | "`outcome'" == "eng10N") {
					quietly: levelsof app_grade, local(grades)
					foreach i in `grades' {
						quietly: levelsof test if app_grade == `i', local(tests)
						foreach x in `tests' {
							capture: replace lambda_`sch' = (1 - abs(runvar_`sch'/bw_`i'_`sch'_`x'))*(abs(runvar_`sch'/bw_`i'_`sch'_`x') <= 1) if app_grade == `i' & test == "`x'"
							capture: replace extrapol_lambda_`sch' = (1 - abs((runvar_`sch' - above_`sch')/bw_`i'_`sch'_`x'))*(abs((runvar_`sch' - above_`sch')/bw_`i'_`sch'_`x') <= 1)	if app_grade == `i' & test == "`x'"
						}
					}
				}

				if (`nyc' == 0 & "`outcome'" != "math7" & "`outcome'" != "eng7" & "`outcome'" != "math8" & "`outcome'" != "eng8") {	
				
					* RF *
				
					ivreg2 score_ above_`sch' `ik_runvar_controls' `fixed_effects' [aw = lambda_`sch'] if disc_`sch' == 1 & experiment_`sch' == 1, `se' partial(`fixed_effects')
					eststo gr`gr'_`outcome'_`sch'_ik

					estwrite _all using "$dir/Paper_Material/Tables/Estimates/rf_`sample'_gr`gr'_`outcome'.sters", append
					est clear				
					
					* Lee (2009) bounds *
					
					su have_ if disc_`sch' == 1 & experiment_`sch' == 1 & above_`sch' == 0
					local p0 = r(mean)
					su have_ if disc_`sch' == 1 & experiment_`sch' == 1 & above_`sch' == 1
					local p1 = r(mean)
					
					local p = 100*(`p1' - `p0')/`p1'
					local invp = 100 - `p'
					
					centile score_ if disc_`sch' == 1 & experiment_`sch' == 1 & above_`sch' == 0, c(`p')
					local y_p = r(c_1)
					centile score_ if disc_`sch' == 1 & experiment_`sch' == 1 & above_`sch' == 1, c(`invp')
					local y_invp = r(c_1)

					gen temp = (above_`sch' == 1 & score_ <= `y_invp') | above_`sch' == 0
					ivreg2 score_ above_`sch' `ik_runvar_controls' `fixed_effects' [aw = lambda_`sch'] if disc_`sch' == 1 & experiment_`sch' == 1 & temp == 1, `se' partial(`fixed_effects')
					scalar lb_gr`gr'_`outcome'_`sch'_ik = _b[above_`sch']
					drop temp
					
					gen temp = (above_`sch' == 1 & score_ >= `y_p') | above_`sch' == 0
					ivreg2 score_ above_`sch' `ik_runvar_controls' `fixed_effects' [aw = lambda_`sch'] if disc_`sch' == 1 & experiment_`sch' == 1 & temp == 1, `se' partial(`fixed_effects')
					scalar ub_gr`gr'_`outcome'_`sch'_ik = _b[above_`sch']
					drop temp									
					
					* Attrition *
		
					if (substr("`outcome'", 1, 5) != "take_") {
						ivreg2 have_ above_`sch' `ik_runvar_controls' `fixed_effects_attrition' [aw = lambda_`sch'] if disc_`sch' == 1 & experiment_`sch' == 1, cluster(id) partial(`fixed_effects_attrition')
						eststo gr`gr'_`outcome'_`sch'_ik
						
						estwrite _all using "$dir/Paper_Material/Tables/Estimates/attrition_`sample'_gr`gr'_`outcome'.sters", append
						est clear
					}
					
					if ("`outcome'" == "math7810" | "`outcome'" == "eng7810") {
						ivreg2 have_ pool*_above_`sch' `ik_runvar_controls' `fixed_effects_attrition' [aw = lambda_`sch'] if disc_`sch' == 1 & experiment_`sch' == 1, cluster(id) partial(`fixed_effects_attrition')	
						testparm pool*_above_`sch' 
						scalar attr_`outcome'_`sch'_ik_f = r(F)
						scalar attr_`outcome'_`sch'_ik_p = r(p)
					}					
				}

				* Parametric estimation *	
				
				if (`nyc' == 0 & "`outcome'" != "math7" & "`outcome'" != "eng7" & "`outcome'" != "math8" & "`outcome'" != "eng8") {	
					
					* RF *
					
					ivreg2 score_ above_`sch' `cbc_runvar_controls' `fixed_effects' if experiment_`sch' == 1 & disc_`sch' == 1, `se' partial(`fixed_effects')
					eststo gr`gr'_`outcome'_`sch'_cbc

					estwrite _all using "$dir/Paper_Material/Tables/Estimates/rf_`sample'_gr`gr'_`outcome'.sters", append
					est clear					
					
					* Lee (2009) bounds *
					
					su have_ if disc_`sch' == 1 & experiment_`sch' == 1 & above_`sch' == 0
					local p0 = r(mean)
					su have_ if disc_`sch' == 1 & experiment_`sch' == 1 & above_`sch' == 1
					local p1 = r(mean)
					
					local p = 100*(`p1' - `p0')/`p1'
					local invp = 100 - `p'
					
					centile score_ if disc_`sch' == 1 & experiment_`sch' == 1 & above_`sch' == 0, c(`p')
					local y_p = r(c_1)
					centile score_ if disc_`sch' == 1 & experiment_`sch' == 1 & above_`sch' == 1, c(`invp')
					local y_invp = r(c_1)

					gen temp = (above_`sch' == 1 & score_ <= `y_invp') | above_`sch' == 0
					ivreg2 score_ above_`sch' `cbc_runvar_controls' `fixed_effects' if experiment_`sch' == 1 & disc_`sch' == 1 & temp == 1, `se' partial(`fixed_effects')
					scalar lb_gr`gr'_`outcome'_`sch'_cbc = _b[above_`sch']
					drop temp
					
					gen temp = (above_`sch' == 1 & score_ >= `y_p') | above_`sch' == 0
					ivreg2 score_ above_`sch' `cbc_runvar_controls' `fixed_effects' if experiment_`sch' == 1 & disc_`sch' == 1 & temp == 1, `se' partial(`fixed_effects')
					scalar ub_gr`gr'_`outcome'_`sch'_cbc = _b[above_`sch']
					drop temp						
					
					* Attrition *
					
					if (substr("`outcome'", 1, 5) != "take_") {
						ivreg2 have_ above_`sch' `cbc_runvar_controls' `fixed_effects_attrition' if experiment_`sch' == 1 & disc_`sch' == 1, cluster(id) partial(`fixed_effects_attrition')
						eststo gr`gr'_`outcome'_`sch'_cbc
						
						estwrite _all using "$dir/Paper_Material/Tables/Estimates/attrition_`sample'_gr`gr'_`outcome'.sters", append
						est clear
					}
					
					if ("`outcome'" == "math7810" | "`outcome'" == "eng7810") {
						ivreg2 have_ pool*_above_`sch' `cbc_runvar_controls' `fixed_effects_attrition' if experiment_`sch' == 1 & disc_`sch' == 1, cluster(id) partial(`fixed_effects_attrition')
						testparm pool*_above_`sch' 
						scalar attr_`outcome'_`sch'_cbc_f = r(F)
						scalar attr_`outcome'_`sch'_cbc_p = r(p)
					}					
				}
			}
			
			* Stack the school-specific samples *

			drop pool*_*runvar_* pool_risk_set_prefs_*
			gen stackid = _n

			reshape long above_ runvar_ runvar2_ runvar3_ intrunvar_ intrunvar2_ intrunvar3_ lambda_ extrapol_lambda_ disc_ experiment_, i(stackid) j(sch) string			
			foreach sch in `schools' {
				gen stc_above_`sch' = above_*(sch == "`sch'")
			}
			
			egen poolstack = group(sch pool)
			quietly: su poolstack
			local min = r(min)
			local max = r(max)
			forval i = `min'/`max' {
				foreach x of varlist runvar*_ intrunvar*_ {				
					gen poolstack`i'_`x' = (poolstack == `i')*`x'
				}
			}
			
			egen poolstack_risk_set_prefs = group(poolstack risk_set_prefs)
			quietly: tab poolstack_risk_set_prefs, gen(poolstack_risk_set_prefs_)
			
			egen poolstack_test_yr = group(poolstack yr_)
			quietly: tab poolstack_test_yr, gen(poolstack_test_yr_)
			
			quietly: su pool
			local min = r(min)
			local max = r(max)
			forval i = `min'/`max' {				
				gen pool`i'_above_ = (pool == `i')*above_	
			}	
			
			gen peergap_pot = .
			gen apprank = .
			foreach sch in `schools' {
				capture: replace peergap_pot = peergap_`outcome'_`sch' if sch == "`sch'"
				capture: replace apprank = apprank_`outcome'_`sch' if sch == "`sch'"
			}
			
			bysort app_grade schyear sch: egen temp1 = mean(apprank) if disc_ == 1 & experiment_ == 1
			bysort app_grade schyear sch: egen temp2 = mean(temp1)
			replace apprank = apprank - temp2
			drop temp1 temp2
			
			gen above_peergap_pot = above_*peergap_pot
			gen to_exam_peergap_pot = to_exam*peergap_pot
			
			gen above_apprank = above_*apprank
			gen to_exam_apprank = to_exam*apprank

			local stc_ik_runvar_controls = "poolstack*_*runvar_"
			local stc_cbc_runvar_controls = "poolstack*_*runvar_ poolstack*_*runvar2_ poolstack*_*runvar3_ "
		
			local stc_fixed_effects = "poolstack_risk_set_prefs_* poolstack_test_yr_*"
			local stc_fixed_effects_attrition = "poolstack_risk_set_prefs_*"
			
			* IK estimation *
						
			if (`nyc' == 0 & "`outcome'" != "math7" & "`outcome'" != "eng7" & "`outcome'" != "math8" & "`outcome'" != "eng8") {	
				ivreg2 score_ above_ `stc_ik_runvar_controls' `stc_fixed_effects' [aw = lambda_] if disc_ == 1 & experiment_ == 1, `se_stc' partial(`stc_fixed_effects')
				eststo gr`gr'_`outcome'_stc_ik

				estwrite _all using "$dir/Paper_Material/Tables/Estimates/rf_`sample'_gr`gr'_`outcome'.sters", append
				est clear
				
				* Lee (2009) bounds *
				
				su have_ if disc_ == 1 & experiment_ == 1 & above_ == 0
				local p0 = r(mean)
				su have_ if disc_ == 1 & experiment_ == 1 & above_ == 1
				local p1 = r(mean)
				
				local p = 100*(`p1' - `p0')/`p1'
				local invp = 100 - `p'
				
				centile score_ if disc_ == 1 & experiment_ == 1 & above_ == 0, c(`p')
				local y_p = r(c_1)
				centile score_ if disc_ == 1 & experiment_ == 1 & above_ == 1, c(`invp')
				local y_invp = r(c_1)

				gen temp = (above_ == 1 & score_ <= `y_invp') | above_ == 0
				ivreg2 score_ above_ `stc_ik_runvar_controls' `stc_fixed_effects' [aw = lambda_] if disc_ == 1 & experiment_ == 1 & temp == 1, `se_stc' partial(`stc_fixed_effects')
				scalar lb_gr`gr'_`outcome'_stc_ik = _b[above_]
				drop temp
			
				gen temp = (above_ == 1 & score_ >= `y_p') | above_ == 0
				ivreg2 score_ above_ `stc_ik_runvar_controls' `stc_fixed_effects' [aw = lambda_] if disc_ == 1 & experiment_ == 1 & temp == 1, `se_stc' partial(`stc_fixed_effects')
				scalar ub_gr`gr'_`outcome'_stc_ik = _b[above_]
				drop temp				
				
				* Attrition *

				if (substr("`outcome'", 1, 5) != "take_") {
					ivreg2 have_ above_ `stc_ik_runvar_controls' `stc_fixed_effects_attrition' [aw = lambda_] if disc_ == 1 & experiment_ == 1, cluster(id) partial(`stc_fixed_effects_attrition')
					eststo gr`gr'_`outcome'_stc_ik
					
					estwrite _all using "$dir/Paper_Material/Tables/Estimates/attrition_`sample'_gr`gr'_`outcome'.sters", append
					est clear	
				}	
			
				if ("`outcome'" == "math7810" | "`outcome'" == "eng7810") {
					ivreg2 have_ pool*_above_ `stc_ik_runvar_controls' `stc_fixed_effects_attrition' [aw = lambda_] if disc_ == 1 & experiment_ == 1, cluster(id) partial(`stc_fixed_effects_attrition')
					testparm pool*_above_
					scalar attr_`outcome'_stc_ik_f = r(F)
					scalar attr_`outcome'_stc_ik_p = r(p)
				}	
			}

			* Parametric estimation *
						
			if (`nyc' == 0 & "`outcome'" != "math7" & "`outcome'" != "eng7" & "`outcome'" != "math8" & "`outcome'" != "eng8") {		
				ivreg2 score_ above_ `stc_cbc_runvar_controls' `stc_fixed_effects' if experiment_ == 1 & disc_ == 1, `se_stc' partial(`stc_fixed_effects')
				eststo gr`gr'_`outcome'_stc_cbc
	
				estwrite _all using "$dir/Paper_Material/Tables/Estimates/rf_`sample'_gr`gr'_`outcome'.sters", append
				est clear			
			
				* Lee (2009) bounds *
				
				su have_ if disc_ == 1 & experiment_ == 1 & above_ == 0
				local p0 = r(mean)
				su have_ if disc_ == 1 & experiment_ == 1 & above_ == 1
				local p1 = r(mean)
				
				local p = 100*(`p1' - `p0')/`p1'
				local invp = 100 - `p'
				
				centile score_ if disc_ == 1 & experiment_ == 1 & above_ == 0, c(`p')
				local y_p = r(c_1)
				centile score_ if disc_ == 1 & experiment_ == 1 & above_ == 1, c(`invp')
				local y_invp = r(c_1)

				gen temp = (above_ == 1 & score_ <= `y_invp') | above_ == 0
				ivreg2 score_ above_ `stc_cbc_runvar_controls' `stc_fixed_effects' if experiment_ == 1 & disc_ == 1 & temp == 1, `se_stc' partial(`stc_fixed_effects')
				scalar lb_gr`gr'_`outcome'_stc_cbc = _b[above_]
				drop temp
				
				gen temp = (above_ == 1 & score_ >= `y_p') | above_ == 0
				ivreg2 score_ above_ `stc_cbc_runvar_controls' `stc_fixed_effects' if experiment_ == 1 & disc_ == 1 & temp == 1, `se_stc' partial(`stc_fixed_effects')
				scalar ub_gr`gr'_`outcome'_stc_cbc = _b[above_]
				drop temp				
				
				* Attrition *
				
				if (substr("`outcome'", 1, 5) != "take_") {
					ivreg2 have_ above_ `stc_cbc_runvar_controls' `stc_fixed_effects_attrition' if experiment_ == 1 & disc_ == 1, cluster(id) partial(`stc_fixed_effects_attrition')
					eststo gr`gr'_`outcome'_stc_cbc
					
					estwrite _all using "$dir/Paper_Material/Tables/Estimates/attrition_`sample'_gr`gr'_`outcome'.sters", append
					est clear
				}
				
				if ("`outcome'" == "math7810" | "`outcome'" == "eng7810") {
					ivreg2 have_ pool*_above_ `stc_cbc_runvar_controls' `stc_fixed_effects_attrition' if experiment_ == 1 & disc_ == 1, cluster(id) partial(`stc_fixed_effects_attrition')
					testparm pool*_above_
					scalar attr_`outcome'_stc_cbc_f = r(F)
					scalar attr_`outcome'_stc_cbc_p = r(p)
				}				
			}
			
			* 2SLS models * 

			if ("`sample'" == "all" & ("`outcome'" == "math7810" | "`outcome'" == "eng7810" | "`outcome'" == "math7810N" | "`outcome'" == "eng7810N")) {	
				
				local instruments = "stc_above_*"
				
				egen ac = group(city schyear app_grade)
				quietly: su ac
				local min = r(min)
				local max = r(max)
				forval i = `min'/`max' {
					foreach x of varlist `instruments' {
						gen ac`i'_`x' = (ac == `i')*`x'
					}
				}
				drop ac
				compress
				
				local ac_instruments = "ac*_stc_above_*"
				
				rename prop_blackhisp prop_
				
				* IK version *

				forval i = 1/6 {
					if `i' == 1 {
						local y = "peer_"
					}
					else if `i' == 2 {
						local y = "prop_"
					}
					else if `i' == 3 {
						local y = "peer_ prop_"
					}
					else if `i' == 4 {
						local y = "peer_ exam_yrs_"
					}
					else if `i' == 5 {
						local y = "prop_ exam_yrs_"
					}
					else if `i' == 6 {
						local y = "peer_ prop_ exam_yrs_"
					}
			
					ivreg2 `y' `instruments' `stc_ik_runvar_controls' `stc_fixed_effects' [aw = lambda_] if experiment_ == 1 & disc_ == 1 & score_ != ., `se_stc' partial(`stc_fixed_effects')
					eststo gr`gr'_`outcome'_ik_fs_`i'
					
					ivreg2 score_ (`y' = `instruments') `stc_ik_runvar_controls' `stc_fixed_effects' [aw = lambda_] if experiment_ == 1 & disc_ == 1, `se_stc' partial(`stc_fixed_effects') ffirst
					eststo gr`gr'_`outcome'_ik_iv_`i'
					foreach x in `y' {
						mat f = e(first)
						mat temp =  f["APF", "`x'"]
						scalar fik_`gr'_`outcome'_`x'_`i' = trace(temp)
					}

					ivreg2 score_ (`y' = `ac_instruments') `stc_ik_runvar_controls' `stc_fixed_effects' [aw = lambda_] if experiment_ == 1 & disc_ == 1, `se_stc' partial(`stc_fixed_effects') ffirst
					eststo gr`gr'_`outcome'_ik_ivac_`i'
					foreach x in `y' {
						mat f = e(first)
						mat temp =  f["APF", "`x'"]
						scalar fik_`gr'_`outcome'_`x'_`i'_ac = trace(temp)
					}
				}

				* IK version: BFLP 1 *
				
				if ("`sample'" == "all" & ("`outcome'" == "math7810" | "`outcome'" == "eng7810")) {				
					ivreg2 score_ (to_exam to_exam_peergap_pot = above_ above_peergap_pot) peergap_pot `stc_ik_runvar_controls' `stc_fixed_effects' [aw = lambda_] if experiment_ == 1 & disc_ == 1, `se_stc' partial(`stc_fixed_effects')
					eststo gr`gr'_`outcome'_ik_gapiv
					
					ivreg2 to_exam above_ above_peergap_pot peergap_pot `stc_ik_runvar_controls' `stc_fixed_effects' [aw = lambda_] if experiment_ == 1 & disc_ == 1 & score_ != ., `se_stc' partial(`stc_fixed_effects')
					eststo gr`gr'_`outcome'_ik_gapfs1
					
					ivreg2 to_exam_peergap_pot above_ above_peergap_pot peergap_pot `stc_ik_runvar_controls' `stc_fixed_effects' [aw = lambda_] if experiment_ == 1 & disc_ == 1 & score_ != ., `se_stc' partial(`stc_fixed_effects')
					eststo gr`gr'_`outcome'_ik_gapfs2
				}
				
				* IK version: BFLP 2 *
				
*				if ("`sample'" == "all" & ("`outcome'" == "math7810" | "`outcome'" == "eng7810")) {				
*					ivreg2 score_ (to_exam to_exam_apprank = above_ above_apprank) apprank `stc_ik_runvar_controls' `stc_fixed_effects' [aw = lambda_] if experiment_ == 1 & disc_ == 1, `se_stc' partial(`stc_fixed_effects')
*					eststo gr`gr'_`outcome'_ik_gapiv
*					
*					ivreg2 to_exam above_ above_apprank apprank `stc_ik_runvar_controls' `stc_fixed_effects' [aw = lambda_] if experiment_ == 1 & disc_ == 1 & score_ != ., `se_stc' partial(`stc_fixed_effects')
*					eststo gr`gr'_`outcome'_ik_gapfs1
*					
*					ivreg2 to_exam_apprank above_ above_apprank apprank `stc_ik_runvar_controls' `stc_fixed_effects' [aw = lambda_] if experiment_ == 1 & disc_ == 1 & score_ != ., `se_stc' partial(`stc_fixed_effects')
*					eststo gr`gr'_`outcome'_ik_gapfs2
*				}
				
				* Parametric version *
				
				forval i = 1/6 {
					if `i' == 1 {
						local y = "peer_"
					}
					else if `i' == 2 {
						local y = "prop_"
					}
					else if `i' == 3 {
						local y = "peer_ prop_"
					}
					else if `i' == 4 {
						local y = "peer_ exam_yrs_"
					}
					else if `i' == 5 {
						local y = "prop_ exam_yrs_"
					}
					else if `i' == 6 {
						local y = "peer_ prop_ exam_yrs_"
					}
					
					ivreg2 `y' `instruments' `stc_cbc_runvar_controls' `stc_fixed_effects' if score_ != . & experiment_ == 1 & disc_ == 1, `se_stc' partial(`stc_fixed_effects')
					eststo gr`gr'_`outcome'_cbc_fs_`i'

					ivreg2 score_ (`y' = `instruments') `stc_cbc_runvar_controls' `stc_fixed_effects' if experiment_ == 1 & disc_ == 1, `se_stc' partial(`stc_fixed_effects') ffirst
					eststo gr`gr'_`outcome'_cbc_iv_`i'
					foreach x in `y' {
						mat f = e(first)
						mat temp =  f["APF", "`x'"]
						scalar fcbc_`gr'_`outcome'_`x'_`i' = trace(temp)
					}

					ivreg2 score_ (`y' = `ac_instruments') `stc_cbc_runvar_controls' `stc_fixed_effects' if experiment_ == 1 & disc_ == 1, `se_stc' partial(`stc_fixed_effects') ffirst
					eststo gr`gr'_`outcome'_cbc_ivac_`i'
					foreach x in `y' {
						mat f = e(first)
						mat temp =  f["APF", "`x'"]
						scalar fcbc_`gr'_`outcome'_`x'_`i'_ac = trace(temp)
					}
				}

				* Parametric version: BFLP 1 *
				
				if ("`sample'" == "all" & ("`outcome'" == "math7810" | "`outcome'" == "eng7810")) {	
					ivreg2 score_ (to_exam to_exam_peergap_pot = above_ above_peergap_pot) peergap_pot `stc_cbc_runvar_controls' `stc_fixed_effects' if experiment_ == 1 & disc_ == 1 & score_ != ., `se_stc' partial(`stc_fixed_effects')
					eststo gr`gr'_`outcome'_cbc_gapiv
					
					ivreg2 to_exam above_ above_peergap_pot peergap_pot `stc_cbc_runvar_controls' `stc_fixed_effects' if experiment_ == 1 & disc_ == 1 & score_ != ., `se_stc' partial(`stc_fixed_effects')
					eststo gr`gr'_`outcome'_cbc_gapfs1
					
					ivreg2 to_exam_peergap_pot above_ above_peergap_pot peergap_pot `stc_cbc_runvar_controls' `stc_fixed_effects' if experiment_ == 1 & disc_ == 1 & score_ != ., `se_stc' partial(`stc_fixed_effects')
					eststo gr`gr'_`outcome'_cbc_gapfs2
				}
				
				* Parametric version: BFLP 2 *
				
*				if ("`sample'" == "all" & ("`outcome'" == "math7810" | "`outcome'" == "eng7810")) {	
*					ivreg2 score_ (to_exam to_exam_apprank = above_ above_apprank) apprank `stc_cbc_runvar_controls' `stc_fixed_effects' if experiment_ == 1 & disc_ == 1 & score_ != ., `se_stc' partial(`stc_fixed_effects')
*					eststo gr`gr'_`outcome'_cbc_gapiv
*					
*					ivreg2 to_exam above_ above_apprank apprank `stc_cbc_runvar_controls' `stc_fixed_effects' if experiment_ == 1 & disc_ == 1 & score_ != ., `se_stc' partial(`stc_fixed_effects')
*					eststo gr`gr'_`outcome'_cbc_gapfs1
*					
*					ivreg2 to_exam_apprank above_ above_apprank apprank `stc_cbc_runvar_controls' `stc_fixed_effects' if experiment_ == 1 & disc_ == 1 & score_ != ., `se_stc' partial(`stc_fixed_effects')
*					eststo gr`gr'_`outcome'_cbc_gapfs2
*				}
				
				estwrite _all using "$dir/Paper_Material/Tables/Estimates/2sls_`sample'_gr`gr'_`outcome'.sters", append
				est clear
			}
			restore
		}		
	}
	
	* Mean tables *
	
	mat kuv = (.)
	foreach gr in 7 9 79 {
		foreach x in math7 math8 math78 math10 math7810 eng7 eng8 eng78 eng10 eng7810 sciehs take_apstem take_psat psat take_sat sat apnbrall apnbr500 apscrall apscr500 attend_any persist_any graduate_any attend_4yr persist_4yr graduate_4yr barrons_C barrons_HC {
			local temp1 = "mean_`x'_gr`gr'"
			local temp2 = "n_`x'_gr`gr'"
			capture: mat rivi = ($`temp1'\ .\ $`temp2'\ .)
			if _rc == 0 {
				capture: mat rownames rivi = "`x'_gr`gr'"
				capture: mat kuv = (kuv\ rivi)
			}
		}
	}
	capture: mat kuv = kuv[2..., 1...]
	mat2txt, matrix(kuv) saving("$dir/Paper_Material/Tables/outcome_means_`sample'.txt") replace

	mat kuv = (.)
	foreach gr in 7 9 79 {
		foreach x in math7 math8 math78 math10 math7810 eng7 eng8 eng78 eng10 eng7810 sciehs {
			local temp1 = "mean_h_`x'_gr`gr'"
			local temp2 = "n_h_`x'_gr`gr'"
			capture: mat rivi = ($`temp1'\ .\ $`temp2'\ .)
			if _rc == 0 {
				capture: mat rownames rivi = "`x'_gr`gr'"
				capture: mat kuv = (kuv\ rivi)
			}
		}
	}
	capture: mat kuv = kuv[2..., 1...]
	capture: mat2txt, matrix(kuv) saving("$dir/Paper_Material/Tables/attrition_means_`sample'.txt") replace

	mat kuv = (.)
	foreach gr in 7 9 79 {
		foreach x in math7 math8 math78 math10 math7810 eng7 eng8 eng78 eng10 eng7810 {
			local temp1 = "mean_base_`x'_gr`gr'"
			local temp2 = "n_base_`x'_gr`gr'"
			capture: mat rivi = ($`temp1'\ .\ $`temp2'\ .)
			if _rc == 0 {
				capture: mat rownames rivi = "`x'_gr`gr'"
				capture: mat kuv = (kuv\ rivi)
			}
		}
	}
	capture: mat kuv = kuv[2..., 1...]
	capture: mat2txt, matrix(kuv) saving("$dir/Paper_Material/Tables/baseline_means_`sample'.txt") replace

	mat kuv = (.)
	foreach gr in 7 9 79 {
		foreach x in math7 math8 math78 math10 math7810 eng7 eng8 eng78 eng10 eng7810 {
			local temp1 = "mean_level_`x'_gr`gr'"
			local temp2 = "n_level_`x'_gr`gr'"
			capture: mat rivi = ($`temp1'\ .\ $`temp2'\ .)
			if _rc == 0 {
				capture: mat rownames rivi = "`x'_gr`gr'"
				capture: mat kuv = (kuv\ rivi)
			}
		}
	}
	capture: mat kuv = kuv[2..., 1...]
	capture: mat2txt, matrix(kuv) saving("$dir/Paper_Material/Tables/level_means_`sample'.txt") replace

	* RF tables *
	
	local options = `"b(3) se(3) keep(est) rename(above_obr est above_bla est above_bls est above_ est) mtitle nogaps compress nonotes nonumbers star(* 0.1 ** 0.05 *** 0.01)"'
	local file = `"$dir/Paper_Material/Tables/rf_table_`sample'.csv"'
	
	est clear
	gen temp = 1
	quietly: reg temp
	esttab using "`file'", nogaps compress nonotes nonumbers replace
	drop temp
	
	local estimates1 = "gr7_math78 gr79_math10 gr79_math7810 gr7_eng78 gr79_eng10 gr79_eng7810 gr7_sciehs gr9_sciehs gr79_sciehs"
	local estimates2 = "gr7_take_psat gr9_take_psat gr79_take_psat gr7_psat gr9_psat gr79_psat"
	local estimates3 = "gr7_take_sat gr9_take_sat gr79_take_sat gr7_sat gr9_sat gr79_sat"
	local estimates4 = "gr7_apnbrall gr9_apnbrall gr79_apnbrall gr7_apscrall gr9_apscrall gr79_apscrall"
	local estimates5 = "gr7_apnbr500 gr9_apnbr500 gr79_apnbr500 gr7_apscrall gr9_apscr500 gr79_apscr500 gr7_take_apstem gr9_take_apstem gr79_take_apstem"
	local estimates6 = "gr7_attend_any gr9_attend_any gr79_attend_any gr7_attend_4yr gr9_attend_4yr gr79_attend_4yr"	
	local estimates7 = "gr7_persist_any gr9_persist_any gr79_persist_any gr7_persist_4yr gr9_persist_4yr gr79_persist_4yr"
	local estimates8 = "gr7_graduate_any gr9_graduate_any gr79_graduate_any gr7_graduate_4yr gr9_graduate_4yr gr79_graduate_4yr"
	local estimates9 = "gr7_barrons_C gr9_barrons_C gr79_barrons_C gr7_barrons_HC gr9_barrons_HC gr79_barrons_HC"
	local estimates10 = "gr7_math10 gr7_eng10 gr9_math10 gr9_eng10"
	
	foreach est in `estimates1' `estimates2' `estimates3' `estimates4' `estimates5' `estimates6' `estimates7' `estimates8' `estimates9' `estimates10' {
		est clear 
		capture: estread _all using "$dir/Paper_Material/Tables/Estimates/rf_`sample'_`est'.sters"	 
		capture: esttab `est'_*_cbc `est'_*_ik using "`file'", `options' append
	}

	* Attrition tables *
	
	local options = `"b(3) se(3) keep(est) rename(above_obr est above_bla est above_bls est above_ est) mtitle nogaps compress nonotes nonumbers star(* 0.1 ** 0.05 *** 0.01)"'
	local file = `"$dir/Paper_Material/Tables/attrition_table_`sample'.csv"'
	
	est clear
	gen temp = 1
	quietly: reg temp
	esttab using  "`file'", nogaps compress nonotes nonumbers replace
	drop temp
	
	local estimates1 = "gr7_math78 gr79_math10 gr79_math7810 gr7_eng78 gr79_eng10 gr79_eng7810 gr7_sciehs gr9_sciehs gr79_sciehs"
	
	foreach est in `estimates1' {
		est clear 
		capture: estread _all using "$dir/Paper_Material/Tables/Estimates/attrition_`sample'_`est'.sters"	 
		capture: esttab `est'_*_cbc `est'_*_ik using "`file'", `options' append
	}
	
	capture: mat kuv1 = (attr_math7810_obr_cbc_p, attr_math7810_bla_cbc_p, attr_math7810_bls_cbc_p, attr_math7810_stc_cbc_p, attr_math7810_obr_ik_p, attr_math7810_bla_ik_p, attr_math7810_bls_ik_p, attr_math7810_stc_ik_p)
	capture: mat kuv2 = (attr_eng7810_obr_cbc_p, attr_eng7810_bla_cbc_p, attr_eng7810_bls_cbc_p, attr_eng7810_stc_cbc_p, attr_eng7810_obr_ik_p, attr_eng7810_bla_ik_p, attr_eng7810_bls_ik_p, attr_eng7810_stc_ik_p)
	capture: mat kuv = (kuv1\ kuv2)
	capture: mat rownames kuv = math7810 eng7810
	capture: mat colnames kuv = "obr_cbc" "bla_cbc" "bls_cbc"  "stc_cbc" "obr_ik" "bla_ik" "bls_ik" "stc_ik"
	capture: mat2txt, matrix(kuv) saving("$dir/Paper_Material/Tables/attrition_joint_test.txt") replace
	
	* 2SLS tables *

	local options = `"mtitle nogaps compress nonotes nonumbers star(* 0.1 ** 0.05 *** 0.01)"'
	local file = `"$dir/Paper_Material/Tables/2sls_table_`sample'.csv"'
	
	est clear
	gen temp = 1
	quietly: reg temp
	esttab using  "`file'", nogaps compress nonotes nonumbers replace
	drop temp
		
	local estimates1 = "gr7_math78 gr7_eng78 gr79_math7810 gr79_eng7810 gr9_MathBG gr9_EXRL gr79_math7810N gr79_eng7810N"
	
	foreach est in `estimates1' {
		est clear 
		capture: estread _all using "$dir/Paper_Material/Tables/Estimates/2sls_`sample'_`est'.sters"

		capture: esttab `est'_*_cbc_iv `est'_*_ik_iv using "`file'", b(3) se(3) keep(to) rename(to_obr to to_bla to to_bls to) `options' append
		capture: esttab `est'_*_cbc_fs `est'_*_ik_fs using "`file'", b(3) se(3) keep(above) rename(above_obr above above_bla above above_bls above) `options' append
		
		capture: esttab `est'_cbc_iv_* `est'_ik_iv_* using "`file'", b(3) se(3) keep(peer_ prop_ to_exam) `options' append
		capture: esttab `est'_cbc_fs_1 `est'_cbc_fs_2 `est'_ik_fs_1 `est'_ik_fs_2 using "`file'", b(3) se(3) keep(stc_above*) `options' append		
		capture: esttab `est'_cbc_ivac_* `est'_ik_ivac_* using "`file'", b(3) se(3) keep(peer_ prop_ to_exam) `options' append
	}
	
	local estimates1 = "gr79_math7810 gr79_eng7810"	
	foreach est in `estimates1' {
		est clear 
		capture: estread _all using "$dir/Paper_Material/Tables/Estimates/2sls_`sample'_`est'.sters"
		capture: esttab `est'_cbc_gapiv `est'_ik_gapiv using "`file'", b(3) se(3) keep(peergap_pot to_exam to_exam_peergap_pot) `options' append
		capture: esttab `est'_cbc_gapfs1 `est'_cbc_gapfs2 `est'_ik_gapfs1 `est'_ik_gapfs2 using "`file'", b(3) se(3) keep(peergap_pot above_ above_peergap_pot) `options' append
	}
	
	local estimates1 = "gr79_math7810 gr79_eng7810"	
	foreach est in `estimates1' {
		est clear 
		capture: estread _all using "$dir/Paper_Material/Tables/Estimates/2sls_`sample'_`est'.sters"
		capture: esttab `est'_cbc_gapiv `est'_ik_gapiv using "`file'", b(3) se(3) keep(apprank to_exam to_exam_apprank) `options' append
		capture: esttab `est'_cbc_gapfs1 `est'_cbc_gapfs2 `est'_ik_gapfs1 `est'_ik_gapfs2 using "`file'", b(3) se(3) keep(apprank above_ above_apprank) `options' append
	}
	
	* Lee (2009) bound tables *
	
	mat results = J(1, 16, .)
	foreach est in gr7_math78 gr7_eng78 gr79_math7810 gr79_eng7810 {
		mat row = J(1, 1, .)
		foreach spec in cbc ik {
			foreach sch in obr bla bls stc {
				mat row = (row, lb_`est'_`sch'_`spec', ub_`est'_`sch'_`spec')
			}
		}
		mat rownames row = `est'
		mat results = (results\ row[1..., 2...])
	}
	mat results = results[2..., 1...]
	local temp1 = "cbc_obr_lb cbc_obr_ub cbc_bla_lb cbc_bla_ub cbc_bls_lb cbc_bls_ub cbc_stc_lb cbc_stc_ub"
	local temp2 = "ik_obr_lb ik_obr_ub ik_bla_lb ik_bla_ub ik_bls_lb ik_bls_ub ik_stc_lb ik_stc_ub"
	mat colnames results = `temp1' `temp2'	
	mat2txt, matrix(results) saving("$dir/Paper_Material/Tables/lee_bounds_`sample'.txt") replace

	* Bandwidth table *
	
	mat kuv = (., ., .)
	foreach gr in 7 9 79 {
		foreach x in math7 math8 math78 math10 math7810 eng7 eng8 eng78 eng10 eng7810 biohs sciehs take_psat psat take_sat sat apnbrall apnbr500 apscrall apscr500 attend_any persist_any graduate_any attend_4yr persist_4yr graduate_4yr barrons_C barrons_HC {
			mat rivi = (.)
			foreach sch in obr bla bls {
				capture: mat rivi = (rivi, bw_`gr'_`sch'_`x')				
			}
			if _rc == 0 {
				capture: mat rivi = rivi[1...,2...]
				capture: mat rownames rivi = "`gr'_`x'"
				capture: mat kuv = (kuv\ rivi)
			}
		}
	}
	capture: mat kuv = kuv[2..., 1...]
	capture: mat colnames kuv = obr bla bls
	capture: mat2txt, matrix(kuv) saving("$dir/Paper_Material/Tables/bandwidths_`sample'.txt") replace

	* First stage F statistic table *
	
	if ("`sample'" == "all") {
		foreach spec in cbc ik {
			foreach ac in 0 1 {
				mat kuv_`spec'_ac`ac' = J(1, 19, .)
				foreach gr in 79 {
					foreach y in math7810 eng7810 math7810N eng7810N {
						mat rivi = (.)
						forval i = 1/6 {
							foreach x in peer_ prop_ exam_yrs_ {
								local names : colnames rivi
								if `ac' == 0 {
									capture: mat rivi = (rivi, round(f`spec'_`gr'_`y'_`x'_`i', 0.1))
								}
								else {
									capture: mat rivi = (rivi, round(f`spec'_`gr'_`y'_`x'_`i'_ac, 0.1))
								}
								if _rc != 0 {
									mat rivi = (rivi, .)
								}
								mat colnames rivi = `names' `x'_`i'
								local colnames : colnames rivi
							}

						}
						mat rownames rivi = `spec'_`gr'_`y'_ac`ac'
						mat colnames kuv_`spec'_ac`ac'  = `colnames'
						mat kuv_`spec'_ac`ac' = (kuv_`spec'_ac`ac'\ rivi)

					}
				}
			}
		}
		mat kuv = (kuv_cbc_ac0[2..., 2...]\ kuv_cbc_ac1[2..., 2...]\ kuv_ik_ac0[2..., 2...]\ kuv_ik_ac1[2..., 2...])
		mat2txt, matrix(kuv) saving("$dir/Paper_Material/Tables/fstats_`sample'.txt") replace
	}
	
}

/* Close log file */

log close

/* Get back the unstacked version of the data */

use "$dir/Data/unstacked_data.dta", clear
desc


/********************************************************/
/* COVARIATE BALANCE (PLUS SCHOOL RETENTION) (TABLE A3) */
/********************************************************/


/* Save unstacked version of the data */

save "$dir/Data/unstacked_data.dta", replace

/* Rename the distance variable */

rename distance_next_school distance

/* Reshape the data and estimate the models */

capture: log close
log using "$dir/Results_and_Notes/covariate balance.log", replace

foreach outcome in female black hisp free lep special distance inexam1 inexam2 inexam3 inexam4 inexam5 inexam6 innext7 innext8 innext9 innext10 innext11 innext12 in9sch10 in9sch11 in9sch12 graderep {
	rename `outcome' score_`outcome'
}
gen id = _n
reshape long score_, i(id) j(covariate) string

gen joint_test = 0
foreach covariate in female black hisp free lep special {
	replace joint_test = 1 if covariate == "`covariate'"
}	
 
est clear
foreach gr in 7 9 {
	if (`gr'== 7) {
		local variables1 = "female black hisp free lep special distance"
		local variables2 = "inexam1 inexam2 inexam3 inexam4 inexam5 inexam6"
		local variables3 = "innext7 innext8 innext9 innext10 innext11 innext12 in9sch10 in9sch11 in9sch12 graderep"
	}
	else {
		local variables1 = "female black hisp free lep special distance"
		local variables2 = "inexam1 inexam2 inexam3 inexam4"
		local variables3 = "innext9 innext10 innext11 innext12 graderep"
	}
	foreach outcome in `variables1' `variables2' `variables3' joint_test {
	
		if  ("`outcome'" == "graderep") {
			local se = "cluster(clusterid)"	
			local stc_se = "cluster(clusterid id)"
		}
		else {
			local se = "robust"
			local stc_se = "cluster(id)"
		}
	
		disp "`gr' - `outcome'"
		
		preserve
		
		if  ("`outcome'" == "joint_test") {
			keep if app_grade == `gr' & joint_test == 1
		}
		else {
			keep if app_grade == `gr' & covariate == "`outcome'"		
		}
		tab covariate
		
		egen clusterid = group(schyear next_school)
		
		egen pool = group(app_grade covariate)
		quietly: su pool
		local min = r(min)
		local max = r(max)
		forval i = `min'/`max' {
			foreach x in runvar runvar2 runvar3 intrunvar intrunvar2 intrunvar3 {
				foreach sch in obr bla bls {
					gen pool`i'_`x'_`sch' = (pool == `i')*`x'_`sch'
				}
			}
		}
		egen pool_risk_set_prefs = group(pool risk_set_pref)
		quietly: tab pool_risk_set_prefs, gen(pool_risk_set_prefs_)
		
		egen temp = group(covariate)
		quietly: su temp
		local min = r(min)
		local max = r(max)		
		forval i = `min'/`max' {
			foreach sch in obr bla bls {			
				gen cov`i'_above_`sch' = (temp == `i')*above_`sch'
			}
		}
		drop temp
		
		* Mean of outcome *
		
		quietly: su score_ if (disc_obr == 1 | disc_bla == 1 | disc_bls == 1)
		global mean_`outcome'_`gr' = r(mean)
		global n_`outcome'_`gr' = r(N)
		
		* School-specific models *
	
		foreach sch in obr bla bls {
		
			local ik_runvar_controls = "pool*_*runvar_`sch'"
			local cbc_runvar_controls = "pool*_*runvar_`sch' pool*_*runvar2_`sch' pool*_*runvar3_`sch'"
			local fixed_effects = "pool_risk_set_prefs_*"
			
			* Bandwidth *
			
			capture: rdob_mod2 score_ runvar_`sch' if experiment_`sch' == 1 & disc_`sch' == 1, c(0) dm(1)
			if _rc == 0 {
				local h_opt = r(h_opt)
			}
			else {
				local h_opt = 10
			}
			scalar bw_`gr'_`sch'_`outcome' = `h_opt'
			gen lambda_`sch' = (1 - abs(runvar_`sch'/`h_opt'))*(abs(runvar_`sch'/`h_opt') <= 1)
			if (`gr' == 79 | "`outcome'" == "joint_test") {
				quietly: levelsof app_grade, local(grades)
				foreach i in `grades' {
					quietly: levelsof covariate if app_grade == `i', local(covariates)
					foreach x in `covariates' {
						capture: replace lambda_`sch' = (1 - abs(runvar_`sch'/bw_`i'_`sch'_`x'))*(abs(runvar_`sch'/bw_`i'_`sch'_`x') <= 1) if app_grade == `i' & covariate == "`x'"
					}
				}
			}
			
			* IK estimation *
			
			if ("`outcome'" != "joint_test") {
				ivreg2 score_ above_`sch' `ik_runvar_controls' `fixed_effects' [aw = lambda_`sch'] if experiment_`sch' == 1 & disc_`sch' == 1, `se'
				eststo gr`gr'_`outcome'_`sch'_ik
			}
			else {
				ivreg2 score_ cov*_above_`sch' `ik_runvar_controls' `fixed_effects' [aw = lambda_`sch'] if experiment_`sch' == 1 & disc_`sch' == 1, `se'
				testparm cov*_above_`sch' 
				scalar gr`gr'_`sch'_ik_f = r(F)
				scalar gr`gr'_`sch'_ik_p = r(p)
			}
	
			* Parametric estimation *
			
			if ("`outcome'" != "joint_test") {
				ivreg2 score_ above_`sch' `cbc_runvar_controls' `fixed_effects' if experiment_`sch' == 1 & disc_`sch' == 1, `se'
				eststo gr`gr'_`outcome'_`sch'_cbc			
			}
			else {
				ivreg2 score_ cov*_above_`sch' `cbc_runvar_controls' `fixed_effects' if experiment_`sch' == 1 & disc_`sch' == 1, `se'
				testparm cov*_above_`sch'
				scalar gr`gr'_`sch'_cbc_f = r(F)
				scalar gr`gr'_`sch'_cbc_p = r(p)
			}
		}
		
		gen stackid = _n
		reshape long above_ runvar_ runvar2_ runvar3_ intrunvar_ intrunvar2_ intrunvar3_ lambda_ disc_ experiment_, i(stackid) j(sch) string
			
		egen poolstack = group(pool sch)
		quietly: su poolstack
		local min = r(min)
		local max = r(max)
		forval i = `min'/`max' {
			foreach x of varlist runvar*_ intrunvar*_ {
				gen poolstack`i'_`x' = (poolstack == `i')*`x'
			}
		}
		
		egen poolstack_risk_set_prefs = group(poolstack risk_set_prefs)
		quietly: tab poolstack_risk_set_prefs, gen(poolstack_risk_set_prefs_)
		
		egen temp = group(covariate)
		quietly: su temp
		local min = r(min)
		local max = r(max)		
		forval i = `min'/`max' {	
			gen cov`i'_above_ = (temp == `i')*above_
		}
		drop temp

		local stc_ik_runvar_controls = "poolstack*_*runvar_"
		local stc_cbc_runvar_controls = "poolstack*_*runvar_ poolstack*_*runvar2_ poolstack*_*runvar3_"
		local stc_fixed_effects = "poolstack_risk_set_prefs_*"
		
		* IK estimation *
 
		if ("`outcome'" != "joint_test") {
			ivreg2 score_ above_ `stc_ik_runvar_controls' `stc_fixed_effects' [aw = lambda_] if experiment_ == 1 & disc_ == 1, `stc_se'
			eststo gr`gr'_`outcome'_stc_ik
		}
		else {
			ivreg2 score_ cov*_above_ `stc_ik_runvar_controls' `stc_fixed_effects' [aw = lambda_] if experiment_ == 1 & disc_ == 1, `stc_se'
			testparm cov*_above_
			scalar gr`gr'_stc_ik_f = r(F)
			scalar gr`gr'_stc_ik_p = r(p)
		}
		
		* Parametric estimation *
		
		if ("`outcome'" != "joint_test") {		
			ivreg2 score_ above_ `stc_cbc_runvar_controls' `stc_fixed_effects' if experiment_ == 1 & disc_ == 1, `stc_se'
			eststo gr`gr'_`outcome'_stc_cbc	
		}
		else {
			ivreg2 score_ cov*_above_ `stc_cbc_runvar_controls' `stc_fixed_effects' if experiment_ == 1 & disc_ == 1, `stc_se'
			testparm cov*_above_ 
			scalar gr`gr'_stc_cbc_f = r(F)
			scalar gr`gr'_stc_cbc_p = r(p)
		}
		restore
	}
}
	
mat kuv = (.)
foreach gr in 7 9 {
	foreach x in female black hisp free lep special distance_next_school inexam1 inexam2 inexam3 inexam4 inexam5 inexam6 innext7 innext8 innext9 innext10 innext11 innext12 in9sch10 in9sch11 in9sch12 graderep {
		local temp1 = "mean_`x'_`gr'"
		local temp2 = "n_`x'_`gr'"
		capture: mat rivi = ($`temp1'\ .\ $`temp2'\ .)
		if _rc == 0 {
			mat rownames rivi = "`x'_gr`gr'"
			mat kuv = (kuv\ rivi)
		}
	}
}
mat kuv = kuv[2..., 1...]
mat2txt, matrix(kuv) saving("$dir/Paper_Material/Tables/covariate_means.txt") replace

mat kuv1 = (gr7_obr_cbc_f, gr7_bla_cbc_f, gr7_bls_cbc_f, gr7_stc_cbc_f, gr7_obr_ik_f, gr7_bla_ik_f, gr7_bls_ik_f, gr7_stc_ik_f)
mat kuv2 = (gr7_obr_cbc_p, gr7_bla_cbc_p, gr7_bls_cbc_p, gr7_stc_cbc_p, gr7_obr_ik_p, gr7_bla_ik_p, gr7_bls_ik_p, gr7_stc_ik_p)
mat kuv3 = (gr9_obr_cbc_f, gr9_bla_cbc_f, gr9_bls_cbc_f, gr9_stc_cbc_f, gr9_obr_ik_f, gr9_bla_ik_f, gr9_bls_ik_f, gr9_stc_ik_f)
mat kuv4 = (gr9_obr_cbc_p, gr9_bla_cbc_p, gr9_bls_cbc_p, gr9_stc_cbc_p, gr9_obr_ik_p, gr9_bla_ik_p, gr9_bls_ik_p, gr9_stc_ik_p)
mat kuv = (kuv1\ kuv2\ kuv3\ kuv4)
mat rownames kuv = "gr7_f" "gr7_p" "gr9_f" "gr9_p"
mat colnames kuv = "obr_cbc" "bla_cbc" "bls_cbc"  "stc_cbc" "obr_ik" "bla_ik" "bls_ik" "stc_ik"
mat2txt, matrix(kuv) saving("$dir/Paper_Material/Tables/covariate_joint_test.txt") replace

local options = `"b(3) se(3) keep(est) rename(above_obr est above_bla est above_bls est above_ est) mtitle nogaps compress nonotes nonumbers star(* 0.1 ** 0.05 *** 0.01)"'
local file = `"$dir/Paper_Material/Tables/covariate_table.csv"'

esttab gr7_*female*_cbc gr7_*female*_ik using "`file'", `options' replace
esttab gr7_*black*_cbc gr7_*black*_ik using "`file'", `options' append
esttab gr7_*hisp*_cbc gr7_*hisp*_ik using "`file'", `options' append
esttab gr7_*free*_cbc gr7_*free*_ik using "`file'", `options' append
esttab gr7_*lep*_cbc gr7_*lep*_ik using "`file'", `options' append
esttab gr7_*special*_cbc gr7_*special*_ik using "`file'", `options' append

esttab gr9_*female*_cbc gr9_*female*_ik using "`file'", `options' append
esttab gr9_*black*_cbc gr9_*black*_ik using "`file'", `options' append
esttab gr9_*hisp*_cbc gr9_*hisp*_ik using "`file'", `options' append
esttab gr9_*free*_cbc gr9_*free*_ik using "`file'", `options' append
esttab gr9_*lep*_cbc gr9_*lep*_ik using "`file'", `options' append
esttab gr9_*special*_cbc gr9_*special*_ik using "`file'", `options' append

capture: esttab gr7_*distance*_cbc gr7_*distance_next_school*_ik using "`file'", `options' append
capture: esttab gr9_*distance*_cbc gr9_*distance_next_school*_ik using "`file'", `options' append

capture: esttab gr7_*inexam1*_cbc gr7_*inexam1*_ik using "`file'", `options' append
capture: esttab gr7_*inexam2*_cbc gr7_*inexam2*_ik using "`file'", `options' append
capture: esttab gr7_*inexam3*_cbc gr7_*inexam3*_ik using "`file'", `options' append
capture: esttab gr7_*inexam4*_cbc gr7_*inexam4*_ik using "`file'", `options' append
capture: esttab gr7_*inexam5*_cbc gr7_*inexam5*_ik using "`file'", `options' append
capture: esttab gr7_*inexam6*_cbc gr7_*inexam6*_ik using "`file'", `options' append

capture: esttab gr9_*inexam1*_cbc gr9_*inexam1*_ik using "`file'", `options' append
capture: esttab gr9_*inexam2*_cbc gr9_*inexam2*_ik using "`file'", `options' append
capture: esttab gr9_*inexam3*_cbc gr9_*inexam3*_ik using "`file'", `options' append
capture: esttab gr9_*inexam4*_cbc gr9_*inexam4*_ik using "`file'", `options' append

capture: esttab gr7_*innext7*_cbc gr7_*innext7*_ik using "`file'", `options' append
capture: esttab gr7_*innext8*_cbc gr7_*innext8*_ik using "`file'", `options' append
capture: esttab gr7_*innext9*_cbc gr7_*innext9*_ik using "`file'", `options' append
capture: esttab gr7_*innext10*_cbc gr7_*innext10*_ik using "`file'", `options' append
capture: esttab gr7_*innext11*_cbc gr7_*innext11*_ik using "`file'", `options' append
capture: esttab gr7_*innext12*_cbc gr7_*innext12*_ik using "`file'", `options' append

capture: esttab gr9_*innext9*_cbc gr9_*innext9*_ik using "`file'", `options' append
capture: esttab gr9_*innext10*_cbc gr9_*innext10*_ik using "`file'", `options' append
capture: esttab gr9_*innext11*_cbc gr9_*innext11*_ik using "`file'", `options' append
capture: esttab gr9_*innext12*_cbc gr9_*innext12*_ik using "`file'", `options' append

capture: esttab gr7_*in9sch10*_cbc gr7_*in9sch10*_ik using "`file'", `options' append
capture: esttab gr7_*in9sch11*_cbc gr7_*in9sch11*_ik using "`file'", `options' append
capture: esttab gr7_*in9sch12*_cbc gr7_*in9sch12*_ik using "`file'", `options' append

capture: esttab gr7_*graderep*_cbc gr7_*graderep*_ik using "`file'", `options' append
capture: esttab gr9_*graderep*_cbc gr9_*graderep*_ik using "`file'", `options' append

log close

/* Get back the unstacked version of the data */

use "$dir/Data/unstacked_data.dta", clear
desc


/***************************************************************************/
/* DESTINATIONS AND SCHOOL CHARACTERISTICS FOR COMPLIERS (TABLES 1, 2, A1) */
/***************************************************************************/


/* Save unstacked version of the data */

save "$dir/Data/unstacked_data.dta", replace

/* Open log */

capture: log close
log using "$dir/Results_and_Notes/destinations and school characteristics.log", replace

/* Modify percents to be between 0 and 1 */

replace p_licensed = p_licensed/100
replace p_core_qualified = p_core_qualified/100

/* Generate age indicators */

gen above_25 = (age_26_32 + age_33_40 + age_41_48 + age_49_56 + age_57_64 + age_64_)/total_fte
gen above_32 = (age_33_40 + age_41_48 + age_49_56 + age_57_64 + age_64_)/total_fte
gen above_40 = (age_41_48 + age_49_56 + age_57_64 + age_64_)/total_fte
gen above_48 = (age_49_56 + age_57_64 + age_64_)/total_fte
gen above_56 = (age_57_64 + age_64_)/total_fte
gen above_64 = (age_64_)/total_fte

/* Generate variable for average baseline score */

egen base_avg = rowmean(base_math base_eng)

/* Estimate the models */

local school_characteristics1 = "peer_math peer_eng peer_q25_math peer_q25_eng peer_q75_math peer_q75_eng pcrank_math pcrank_eng"
local school_characteristics2 = "prop_blackhisp prop_free prop_female st_ratio p_licensed n_core p_core_qualified above_40 above_48 above_56"

local complier_characteristics = "female black hisp free lep special base_math base_eng"

local samples = "all upperhalf upperquart"

foreach sample in `samples' {
	foreach gr in 7 9 {

		preserve
		keep if app_grade == `gr'
		
		* Full sample vs. subsample selection *
		
		if ("`sample'" == "male") {
			keep if (female == 0)
		}
		else if ("`sample'" == "female") {
			keep if (female == 1)
		}
		else if ("`sample'" == "black") {
			keep if (black == 1)
		}
		else if ("`sample'" == "hisp") {
			keep if (hisp == 1)
		}
		else if ("`sample'" == "blackhisp") {
			keep if (black == 1 | hisp == 1)
		}
		else if ("`sample'" == "nonblackhisp") {
			drop if (black == 1 | hisp == 1)
		}
		else if ("`sample'" == "upperhalf") {
			bysort app_grade schyear: egen temp = pctile(base_avg), p(50)
			keep if (base_avg >= temp & base_avg != .)
			drop temp
		}
		else if ("`sample'" == "upperquart") {
			bysort app_grade schyear: egen temp = pctile(base_avg), p(75)
			keep if (base_avg >= temp & base_avg != .)
			drop temp
		}
		else if ("`sample'" == "lowerhalf") {
			bysort app_grade schyear: egen temp = pctile(base_avg), p(50)
			keep if (base_avg < temp & base_avg != .)
			drop temp
		}
		else if ("`sample'" == "lowerquart") {
			bysort app_grade schyear: egen temp = pctile(base_avg), p(25)
			keep if (base_avg < temp & base_avg != .)
			drop temp
		}
		quietly: count
		disp _newline "Sample size: " r(N) _newline
				
		mat results_dest_all_`gr' = J(4, 1, .)
		mat results_dest_comp_`gr' = J(4, 1, .)
		mat results_schoolchar_comp_`gr' = J(18, 1, .)
		mat results_compchar_`gr' = J(8, 1, .)
		
		quietly: tab risk_set_prefs, gen(risk_set_prefs_)
			
		foreach sch in obr bla bls {
		
			local ik_runvar_controls = "*runvar_`sch'"
			local fixed_effects = "risk_set_prefs_*"
			
			* School distribution on both sides of the cutoff (no fixed effects here) *
		
			mat column = (., .)
			foreach j in bps obr bla bls {
				capture: rdob_mod2 to_`j' runvar_`sch' if experiment_`sch' == 1 & disc_`sch' == 1, c(0) dm(1)
				local h = r(h_opt)
				disp "`sch' - `j' - `h'"
				if (`h' == .) {
					local h = 10
				}
				capture: drop w
				gen w = (1 - abs(runvar_`sch'/`h'))*(abs(runvar_`sch'/`h') <= 1)
				capture noisily: ivreg2 to_`j' above_`sch' `ik_runvar_controls' [aw = w] if experiment_`sch' == 1 & disc_`sch' == 1, r
				if (_rc == 0) {
					mat column = (column\ (_b[_cons], _b[_cons] + _b[above_`sch']))
				}
				else {
					mat column = (column\ (., .))
				}	
			}
			mat results_dest_all_`gr' = (results_dest_all_`gr', column[2..., 1...])
			
			* School distribution for compliers *
					
			capture: drop invd
			gen invd = 1 - to_`sch'
			mat column = (.)
			foreach j in bps obr bla bls {
				capture: drop invd_`j'
				gen invd_`j' = invd*to_`j'
				capture: rdob_mod2 invd_`j' runvar_`sch' if experiment_`sch' == 1 & disc_`sch' == 1, c(0) dm(1)
				local h = r(h_opt)
				disp "`sch' - `j' - `h'"
				if (`h' == .) {
					local h = 10
				}
				capture: drop w
				gen w = (1 - abs(runvar_`sch'/`h'))*(abs(runvar_`sch'/`h') <= 1)
				capture noisily: ivreg2 invd_`j' (invd = above_`sch') `ik_runvar_controls' `fixed_effects' [aw = w] if experiment_`sch' == 1 & disc_`sch' == 1, r
				if (_rc == 0) {
					mat column = (column\ _b[invd])
				}
				else {
					mat column = (column\ .)
				}
				drop invd_`j' w
			}
			mat results_dest_comp_`gr' = (results_dest_comp_`gr', column[2..., 1...])
			drop invd
			
			* School characteristics for compliers *
					
			capture: drop d invd
			gen d = to_`sch'
			gen invd = 1 - to_`sch'
			mat column = (., .)
			foreach j in `school_characteristics1' `school_characteristics2' {
				mat row = (.)
				
				gen invd_`j' = invd*`j'
				capture: rdob_mod2 invd_`j' runvar_`sch' if experiment_`sch' == 1 & disc_`sch' == 1, c(0) dm(1)
				local h = r(h_opt)
				disp "`sch' - `j' - `h'"
				if (`h' == .) {
					local h = 10
				}
				capture: drop w
				gen w = (1 - abs(runvar_`sch'/`h'))*(abs(runvar_`sch'/`h') <= 1)
				capture noisily: ivreg2 invd_`j' (invd = above_`sch') `ik_runvar_controls' `fixed_effects' [aw = w] if experiment_`sch' == 1 & disc_`sch' == 1, r
				if (_rc == 0) {
					mat row = (row, _b[invd])
				}
				else {
					mat row = (row, .)
				}
				drop invd_`j' w
				
				capture: drop d_`j'
				gen d_`j' = d*`j'
				capture: rdob_mod2 d_`j' runvar_`sch' if experiment_`sch' == 1 & disc_`sch' == 1, c(0) dm(1)
				local h = r(h_opt)
				disp "`sch' - `j' - `h'"
				if (`h' == .) {
					local h = 10
				}
				capture: drop w
				gen w = (1 - abs(runvar_`sch'/`h'))*(abs(runvar_`sch'/`h') <= 1)
				capture noisily: ivreg2 d_`j' (d = above_`sch') `ik_runvar_controls' `fixed_effects' [aw = w] if experiment_`sch' == 1 & disc_`sch' == 1, r
				if (_rc == 0) {
					mat row = (row, _b[d])
				}
				else {
					mat row = (row, .)
				}
				drop d_`j' w
				
				mat column = (column\ row[1..., 2...])
				
			}
			mat results_schoolchar_comp_`gr' = (results_schoolchar_comp_`gr', column[2..., 1...])
			drop d invd
			
			* Compliers characteristics *
					
			mat column = (.)
			foreach j in `complier_characteristics' {
			
				capture: drop d_`j'
				gen d_`j' = to_`sch'*`j'
				capture: rdob_mod2 d_`j' runvar_`sch' if experiment_`sch' == 1 & disc_`sch' == 1, c(0) dm(1)
				local h = r(h_opt)
				disp "`sch' - `j' - `h'"
				if (`h' == .) {
					local h = 10
				}
				capture: drop w
				gen w = (1 - abs(runvar_`sch'/`h'))*(abs(runvar_`sch'/`h') <= 1)
				capture noisily: ivreg2 d_`j' (to_`sch' = above_`sch') `ik_runvar_controls' `fixed_effects' [aw = w] if experiment_`sch' == 1 & disc_`sch' == 1, r
				if (_rc == 0) {
					mat column = (column\ _b[to_`sch'])
				}
				else {
					mat column = (column\ .)
				}
				drop d_`j' w

			}
			mat results_compchar_`gr' = (results_compchar_`gr', column[2..., 1...])
		
		}
		
		mat results_dest_all_`gr' = results_dest_all_`gr'[1..., 2...]
		mat colnames results_dest_all_`gr' = "all_obr0" "all_obr1" "all_bla0" "all_bla1" "all_bls0" "all_bls1"
		mat rownames results_dest_all_`gr' = "bps" "obr" "bla" "bls"
		
		mat results_dest_comp_`gr' = results_dest_comp_`gr'[1..., 2...]
		mat colnames results_dest_comp_`gr' = "comp_obr" "comp_bla" "comp_bls"
		mat rownames results_dest_comp_`gr' = "bps" "obr" "bla" "bls"	
		
		mat results_schoolchar_comp_`gr' = results_schoolchar_comp_`gr'[1..., 2...]
		mat colnames results_schoolchar_comp_`gr' = "comp_obr0" "comp_obr1" "comp_bla0" "comp_bla1" "comp_bls0" "comp_bls1"
		mat rownames results_schoolchar_comp_`gr' = `school_characteristics1' `school_characteristics2'

		mat results_compchar_`gr' = results_compchar_`gr'[1..., 2...]
		mat colnames results_compchar_`gr' = "comp_obr" "comp_bla" "comp_bls"
		mat rownames results_compchar_`gr' = `complier_characteristics'
		
		restore
	}

	mat results_7 = (results_dest_all_7, results_dest_comp_7)
	mat results_9 = (results_dest_all_9, results_dest_comp_9)
	mat results = (results_7\ results_9)
	mat list results
	mat2txt, matrix(results) saving("$dir/Paper_Material/Tables/school_destinations_`sample'.txt") replace

	mat results = (results_schoolchar_comp_7\ results_schoolchar_comp_9)
	mat list results
	mat2txt, matrix(results) saving("$dir/Paper_Material/Tables/school_characteristics_`sample'.txt") replace
	
	mat results = (results_compchar_7\ results_compchar_9)
	mat list results
	mat2txt, matrix(results) saving("$dir/Paper_Material/Tables/complier_characteristics_`sample'.txt") replace

}

log close

/* Get back the unstacked version of the data */

use "$dir/Data/unstacked_data.dta", clear
desc
